﻿/*
 * mTools - © EUROPEAN COMMISSION, 2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous
 * la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou 
 * implicites.
 * Consultez la Licence pour les autorisations et les restrictions 
 * linguistiques spécifiques relevant de la Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MetaNull.Console
{
	/// <summary>
	/// Helper class for multi threaded application synchronizing writes to the standard console output
	/// </summary>
	public class SynchronizedConsole
	{
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		public static void Write( string message ) {
			lock (UniqueLock.Lock)
			{
				System.Console.Write(message);
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="blocks"></param>
		public static void Write(MessageBlock[] blocks)
		{
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor, b = System.Console.BackgroundColor;
				System.Console.ForegroundColor = f;
				System.Console.BackgroundColor = b;
				foreach (MessageBlock block in blocks)
				{
					System.Console.ForegroundColor = block.foreground;
					System.Console.BackgroundColor = block.background;
					System.Console.Write(block.message);
				}
				System.Console.ForegroundColor = f;
				System.Console.BackgroundColor = b;
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		/// <param name="foreground"></param>
		public static void Write( string message, ConsoleColor foreground ) {
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor;
				System.Console.ForegroundColor = foreground;
				System.Console.Write(message);
				System.Console.ForegroundColor = f;
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		/// <param name="foreground"></param>
		/// <param name="background"></param>
		public static void Write( string message, ConsoleColor foreground, ConsoleColor background ) {
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor, b = System.Console.BackgroundColor;
				System.Console.ForegroundColor = foreground;
				System.Console.BackgroundColor = background;
				System.Console.Write(message);
				System.Console.ForegroundColor = f;
				System.Console.BackgroundColor = b;
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		public static void WriteLine( string message ) {
			lock (UniqueLock.Lock)
			{
				System.Console.WriteLine(message);
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="blocks"></param>
		public static void WriteLine(MessageBlock[] blocks)
		{
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor, b = System.Console.BackgroundColor;
				foreach (MessageBlock block in blocks)
				{
					System.Console.ForegroundColor = block.foreground;
					System.Console.BackgroundColor = block.background;
					System.Console.WriteLine(block.message);
				}
				System.Console.ForegroundColor = f;
				System.Console.BackgroundColor = b;
			}
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		/// <param name="foreground"></param>
		public static void WriteLine( string message, ConsoleColor foreground ) {
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor;
				System.Console.ForegroundColor = foreground;
				System.Console.WriteLine(message);
				System.Console.ForegroundColor = f;
			}
			
		}
		/// <summary>
		/// Write to the console
		/// </summary>
		/// <param name="message"></param>
		/// <param name="foreground"></param>
		/// <param name="background"></param>
		public static void WriteLine(string message, ConsoleColor foreground, ConsoleColor background)
		{
			lock (UniqueLock.Lock)
			{
				ConsoleColor f = System.Console.ForegroundColor, b = System.Console.BackgroundColor;
				System.Console.ForegroundColor = foreground;
				System.Console.BackgroundColor = background;
				System.Console.WriteLine(message);
				System.Console.ForegroundColor = f;
				System.Console.BackgroundColor = b;
			}
		}

		/// <summary>
		/// Holds a Message Block
		/// </summary>
		public sealed class MessageBlock {
			/// <summary>
			/// Message
			/// </summary>
			public string message;
			/// <summary>
			/// Foreground color
			/// </summary>
			public ConsoleColor foreground;
			/// <summary>
			/// Background color
			/// </summary>
			public ConsoleColor background;

			/// <summary>
			/// Constructor
			/// </summary>
			public MessageBlock() {
				message = string.Empty;
				foreground = ConsoleColor.White;
				background = ConsoleColor.Black;
			}
			/// <summary>
			/// Constructor
			/// </summary>
			/// <param name="message"></param>
			/// <param name="fore"></param>
			/// <param name="back"></param>
			public MessageBlock( string message, ConsoleColor fore, ConsoleColor back ) {
				this.message = message;
				this.foreground = fore;
				this.background = back;
			}
		}

		private SynchronizedConsole()
		{
		}

		/// <summary>
		/// Synchronization object
		/// </summary>
		private sealed class UniqueLock
		{
			/// <summary>
			/// Accessor to the unique instance of the Lock object
			/// </summary>
			public static object Lock
			{
				get
				{
					return Singleton.TheOnlyLockInstance;
				}
			}
			/// <summary>
			/// Unique instance of the lock object
			/// </summary>
			private object TheOnlyLockInstance;

			/// <summary>
			/// Instanciation of the static singleton object
			/// </summary>
			private static readonly UniqueLock Singleton = new UniqueLock();

			/// <summary>
			/// Initialization of the lock object
			/// </summary>
			private UniqueLock()
			{
				TheOnlyLockInstance = new object();
			}
		}
	}
}
