﻿/*
 * mTools - © EUROPEAN COMMISSION, 2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous
 * la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou 
 * implicites.
 * Consultez la Licence pour les autorisations et les restrictions 
 * linguistiques spécifiques relevant de la Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;

using System.Xml.Linq;

using System.Resources;
using System.Globalization;

namespace MetaNull.Xml
{
	/// <summary>
	/// Class meant to hold compiled stylesheets, each as a singleton (only one
	/// instance of the object for the whole process)
	/// </summary>
	/// <remarks>
	/// .NET's XslCompiledTransform() creates and loads one Assembly for each 
	/// XSL sheet loaded... 
	/// so better to load it only once to avoid memory overflow
	/// After processing, you should delete the temporary assemblies using the
	/// XslCompiledTransform.TemporaryFiles list
	/// </remarks>
	public sealed class CompiledXslFiles
	{
		/// <summary>
		/// Accessors for the compiled stylesheets
		/// </summary>
		public static Dictionary<string,XslCompiledTransform> Stylesheets
		{
			get
			{
				lock (UniqueInstance.UniqueLock)
				{
					return UniqueInstance.UniqueStylesheets;
				}
			}
		}
		/// <summary>
		/// Accessor for the compiled stylesheets
		/// </summary>
		/// <param name="key">Stylesheet's key name</param>
		/// <returns></returns>
		public static XslCompiledTransform GetStylesheet(string key)
		{
			lock (UniqueInstance.UniqueLock)
			{
				return UniqueInstance.UniqueStylesheets[key];
			}
		}
		/// <summary>
		/// Load a stylesheet from a resource file (the resource name will be used as the key)
		/// </summary>
		/// <param name="resourceName">Resource name</param>
		public static void LoadResource(string resourceName)
		{
			XmlDocument xsl = Tools.LoadXmlResource(resourceName);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(resourceName, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[resourceName].Load(xsl);
			}
		}
		/// <summary>
		/// Load a stylesheet from a file
		/// </summary>
		/// <param name="key">Key name</param>
		/// <param name="stream">File stream</param>
		public static void Load(string key, Stream stream)
		{
			XmlDocument xsl = new XmlDocument();
			xsl.Load(stream);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(key, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[key].Load(xsl);
			}
		}
		/// <summary>
		/// Load a stylesheet from a file
		/// </summary>
		/// <param name="key">Key name</param>
		/// <param name="fileName">File path</param>
		public static void Load(string key, string fileName)
		{
			XmlDocument xsl = new XmlDocument();
			xsl.Load(fileName);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(key, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[key].Load(xsl);
			}
		}
		/// <summary>
		/// Load a stylesheet from a textreader
		/// </summary>
		/// <param name="key">Key name</param>
		/// <param name="reader">Text reader</param>
		public static void Load(string key, TextReader reader)
		{
			XmlDocument xsl = new XmlDocument();
			xsl.Load(reader);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(key, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[key].Load(xsl);
			}
		}
		/// <summary>
		/// Load a stylesheet from an XmlReader
		/// </summary>
		/// <param name="key">Key name</param>
		/// <param name="reader">Xml reader</param>
		public static void Load(string key, XmlReader reader)
		{
			XmlDocument xsl = new XmlDocument();
			xsl.Load(reader);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(key, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[key].Load(xsl);
			}
		}
		/// <summary>
		/// Load a stylesheet from a astring
		/// </summary>
		/// <param name="key">Key name</param>
		/// <param name="xml">Xml string</param>
		public static void LoadXml(string key, string xml)
		{
			XmlDocument xsl = new XmlDocument();
			xsl.LoadXml(xml);
			lock (UniqueInstance.UniqueLock)
			{
				UniqueInstance.UniqueStylesheets.Add(key, new XslCompiledTransform(fDebug));
				UniqueInstance.UniqueStylesheets[key].Load(xsl);
			}
		}
		/// <summary>
		/// Unload one compiled stylesheet
		/// </summary>
		/// <param name="key">Key name</param>
		public static void Unload(string key)
		{
			lock (UniqueInstance.UniqueLock)
			{
				if (UniqueInstance.UniqueStylesheets.ContainsKey(key))
				{
					UniqueInstance.UniqueStylesheets[key].TemporaryFiles.Delete();
					UniqueInstance.UniqueStylesheets.Remove(key);
				}
				else
				{
					throw new IndexOutOfRangeException();
				}
			}
		}
		/// <summary>
		/// Remove all compiled stylesheets
		/// </summary>
		public static void Clear()
		{
			lock (UniqueInstance.UniqueLock)
			{
				foreach (var pair in UniqueInstance.UniqueStylesheets)
				{
					pair.Value.TemporaryFiles.Delete();
				}
				UniqueInstance.UniqueStylesheets.Clear();
			}
		}

		private Dictionary<string,XslCompiledTransform> UniqueStylesheets;
		private object UniqueLock;
		private static readonly CompiledXslFiles UniqueInstance = new CompiledXslFiles();
		private CompiledXslFiles()
		{
			UniqueStylesheets = new Dictionary<string, XslCompiledTransform>();
			UniqueLock = new object();
		}

#if DEBUG
		internal const bool fDebug = true;
#else
		internal const bool fDebug = false;
#endif
	}
}
