/*
* mUDFDownloader - Copyright 2010 the European Commission
* 
* Licensed under the EUPL (the Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
* http://ec.europa.eu/idabc/eupl.html
* 
* Unless required by applicable law or agreed to in
  writing, software distributed under the Licence is
  distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
  express or implied.
* See the Licence for the specific language governing
  permissions and limitations under the Licence.
*
* The software binaries include MySQL-C binaries (compiled from the un-modified
  MySQL-C source). 
* MySQL-C is copyrighted by Oracle Corp. and licensed under the Gnu GPL v 2.
*
*/
/*
 * Author: Pascal Havelange (Pascal.HAVELANGE@ec.europa.eu)
 */

#ifndef __METANULL_WIN32_H__
#define __METANULL_WIN32_H__


#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdarg.h>

#include <windows.h>

/**
 * Allocate memory in the Process' heap (initializing it to 0)
 * @param SIZE_T cbBytes Number of bytes to allocate
 * @return LPVOID Pointer to the allocated buffer or 0 in case of error
 */
LPVOID mHeapAlloc( SIZE_T cbBytes );

/**
 * ReAllocate memory in the Process' heap (initializing additional bytes to 0)
 * @param LPVOID * lpBuffer Pointer to the currently allocated block of memory (in case of success, it might be updated)
 * @param SIZE_T cbBytes Number of bytes to allocate
 * @return BOOL TRUE in case of success, FALSE otherwise
 */
BOOL mHeapReAlloc( LPVOID * lpBuffer, SIZE_T cbBytes );

/**
 * Free memory allocated by m_alloc
 * @param LPVOID lpBuffer Pointer to the allocated buffer
 * @return BOOL TRUE on success, FALSE otherwise
 */
BOOL mHeapFree( LPVOID lpBuffer );

/**
 * Returns the size of a memory block allocated through m_alloc
 * @param LPVOID lpBuffer Pointer to the allocated buffer
 * @return SIZE_T size of the block, or (SIZE_T)-1 in case of error
 */
SIZE_T mHeapSize( LPVOID lpBuffer );

/**
 * Get the fully qualified name of a path
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszPath Path to analyse
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @param LPTSTR * lpFilePart A pointer to a buffer that receives the address (in lpBuffer) of the final file name component in the path. Specify NULL if you do not need to receive this information. If lpBuffer points to a directory and not a file, lpFilePart receives zero (0).
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mRealPath( LPCTSTR pszPath, LPDWORD pcbPath, LPTSTR * lpFilePart );

/**
 * Get the folder name out of a path name
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszPath Path to analyse
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mDirname( LPCTSTR pszPath, LPDWORD pcbPath );

/**
 * Get the file name out of a path name
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszPath Path to analyse
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mFilename( LPCTSTR pszPath, LPDWORD pcbPath );

/**
 * Get the fully qualified name of the Windows' system directory (C:\Windows\System32)
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mSystemDirectory( LPDWORD pcbPath );

/**
 * Get the fully qualified name of the Windows' windows directory (C:\Windows)
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mWindowsDirectory( LPDWORD pcbPath );

/**
 * Get the fully qualified name of the current Working directory (.\)
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPDWORD pcbPath If != 0, will receive the size of the fully qualified path in TCHARs
 * @return LPTSTR Returns the allocated buffer containing the path, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mCurrentDirectory( LPDWORD pcbPath );

/**
 * Get the message from the last windows error
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPDWORD pcbMessage If != 0, will receive the size of the message
 * @return LPTSTR Returns the allocated buffer containing the message, or 0 in case of error
 * @see mHeapFree()
 */
LPTSTR mGetLastErrorMessage( LPDWORD pcbMessage );

/**
 * Formats a string, allocating necessary memory
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszFormat The format string (see FormatMessage and printf for more information about passing inserts and formating output of inserted data)
 * @param LPDWORD pcbMessage If != 0, will receive the size of the message
 * @return LPTSTR Returns the allocated buffer containing the message, or 0 in case of error
 * @see mHeapFree()
 * @see FormatMessage
 * @see printf
 */
LPTSTR mFormatMessage( LPCTSTR pszFormat, LPDWORD pcbMessage, ... );


/**
 * Returns a string composed of the concatenation of two string. required memory is allocated by the function
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszStr1 The first string to copy
 * @param LPCTSTR pszStr2 The second string to copy
 * @param LPDWORD pcbMessage If != 0, will receive the size of the message
 * @param LPDWORD pcbMessage If != 0, will receive the length of the resulting string in number of TCHARs
 * @return LPTSTR The resulting string
 * @see mHeapFree
 */
LPTSTR mConcatenate( LPCTSTR pszStr1, LPCTSTR pszStr2, LPDWORD pcbMessage );

/**
 * Appends a string to an other, required memory is allocated by the function. Attention, the pointer of the first string is modified. The memory it was containing is freeded using mHeapFree
 * ATTENTION: The pszStr1 pointer is automatically freed and receives the address of a new buffer
 * ATTENTION: The resulting pszStr1 pointer should still be freed using mHeapFree()
 * @param LPTSTR * pszStr1 IN: The first string, OUT: The resulting string
 * @param LPCTSTR pszStr2 The second string to copy
 * @param LPDWORD pcbMessage If != 0, will receive the size of the message
 * @param LPDWORD pcbMessage If != 0, will receive the length of the resulting string in number of TCHARs
 * @return BOOL The resulting string
 * @see mHeapFree
 */
BOOL mConcatenateInPlace( LPTSTR * pszStr1, LPCTSTR pszStr2, LPDWORD pcbMessage );

/**
 * Returns an allocated buffer, initialized with a specific string value
 * ATTENTION: The returned pointer should be freed using mHeapFree()
 * @param LPCTSTR pszStr The string to copy
 * @param DWORD cbStr The length of the string
 * @return LPTSTR The resulting string
 * @see mHeapFree
 */
LPTSTR mString( LPCTSTR pszStr, DWORD cbStr );


#endif // __METANULL_WIN32_H__