﻿/*
 * mAggregatorStatus - © EUROPEAN COMMISSION, 2009-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la
«Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que
conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse
suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le
logiciel distribué sous la Licence est distribué «en
l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT,
expresses ou implicites.
 * Consultez la Licence pour les autorisations et les
restrictions linguistiques spécifiques relevant de la
Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;


namespace mAggregatorService
{
	/// <summary>
	/// Set of counters
	/// </summary>
	/// <remarks>
	/// Author: Pascal (Meta) Havelange
	/// Copyright (C) 2010
	/// </remarks>
	[Serializable()]
	public class mCounterSet 
		/*: ISerializable*/
	{
		/// <summary>
		/// Object for synchronizing access to the counters table
		/// </summary>
		private object counterLock = new object();
		/// <summary>
		/// The list table of counters
		/// </summary>
		private volatile Hashtable _counters = new Hashtable();

		/// <summary>
		/// Constructor
		/// </summary>
		public mCounterSet()
		{
		}
		/// <summary>
		/// Constructor
		/// </summary>
		public mCounterSet(Hashtable t)
		{
			_counters = t;
		}

		/// <summary>
		/// Fetch the list of counters
		/// </summary>
		/// <remarks>
		/// Please us GetCountersCopy instead if you whish to work with the full set of 
		/// counters (otherwise, there is a risk that the counters will be  modified while 
		/// you are browsing the collection)
		/// </remarks>
		public Hashtable Counters
		{
			get
			{
				lock (counterLock)
				{
					return _counters;
				}
			}
		}
		/// <summary>
		/// Increase counter's value by 1 unit
		/// </summary>
		/// <param name="name"></param>
		public void Increment(String name)
		{
			Increment(name, 1);
		}
		/// <summary>
		/// Increase counter's value by a specific value unit
		/// </summary>
		/// <param name="name"></param>
		/// <param name="nb"></param>
		public void Increment(String name, int nb)
		{
			lock (counterLock)
			{
				if (Counters.ContainsKey(name))
				{
					Counters[name] = (Int32)((Int32)Counters[name] + nb);
				}
				else
				{
					Counters[name] = (Int32)nb;
				}
			}
		}
		/// <summary>
		/// Decrease counter's value by 1 unit
		/// </summary>
		/// <param name="name"></param>
		public void Decrement(String name)
		{
			Decrement(name, 1);
		}
		/// <summary>
		/// Decrease counter's value by a specific value unit
		/// </summary>
		/// <param name="name"></param>
		/// <param name="nb"></param>
		public void Decrement(String name, int nb)
		{
			lock (counterLock)
			{
				if (Counters.ContainsKey(name))
				{
					Counters[name] = (Int32)((Int32)Counters[name] - nb);
				}
				else
				{
					Counters[name] = (Int32)(-nb);
				}
			}
		}
		/// <summary>
		/// Set the value of a counter
		/// </summary>
		/// <param name="name"></param>
		/// <param name="value"></param>
		public void Set(String name, int value)
		{
			lock (counterLock)
			{
				if (Counters.ContainsKey(name))
				{
					Counters[name] = value;
				}
			}
		}
		/// <summary>
		/// Get the value of a counter
		/// </summary>
		/// <param name="name"></param>
		/// <returns></returns>
		public int Get(String name)
		{
			if (Counters.ContainsKey(name))
			{
				return 0;
			}
			return (Int32)Counters[name];
		}

		/// <summary>
		/// Returns a copy of the table of counters
		/// </summary>
		/// <returns></returns>
		public Hashtable GetCountersCopy()
		{
			lock (counterLock)
			{
				Hashtable copy = new Hashtable();
				foreach (DictionaryEntry entry in _counters)
				{
					copy.Add(entry.Key, entry.Value);
				}
				return copy;
			}
		}
		/// <summary>
		/// Returns a displayable version of the Hashtable
		/// </summary>
		/// <returns></returns>
		public override string ToString()
		{
			StringBuilder sb = new StringBuilder();
			foreach (DictionaryEntry counter in GetCountersCopy())
			{
				sb.AppendLine(counter.Key + ": " + counter.Value);
			}
			return sb.ToString();
		}

		/*
		#region Implements ISerializable
		/// <summary>
		/// Serialization constructor
		/// </summary>
		/// <param name="info"></param>
		/// <param name="ctxt"></param>
		public mCounterSet(SerializationInfo info, StreamingContext ctxt)
		{
			_counters = (Hashtable)info.GetValue("Counters", typeof(Hashtable));
		}
		/// <summary>
		/// Serialization info
		/// </summary>
		/// <param name="info"></param>
		/// <param name="ctxt"></param>
		public void GetObjectData(SerializationInfo info, StreamingContext ctxt)
		{
			info.AddValue("Counters", _counters);
		}
		#endregion
		*/
	}
}
