﻿/*
 * mAggregatorStatus - © EUROPEAN COMMISSION, 2009-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la
«Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que
conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse
suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le
logiciel distribué sous la Licence est distribué «en
l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT,
expresses ou implicites.
 * Consultez la Licence pour les autorisations et les
restrictions linguistiques spécifiques relevant de la
Licence.
 *
 * Auteur de l'oeuvre: Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *
 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.XPath;
using System.Xml.Linq;
using System.Globalization;

namespace mAggregatorService
{
	public class mXslTools
	{

		/// <summary>
		/// Remove illegal XML characters from a string.
		/// </summary>
		public static string SanitizeXmlString(string xml)
		{
			if (xml == null)
			{
				throw new ArgumentNullException("xml");
			}
		 
			StringBuilder buffer = new StringBuilder(xml.Length);
		 
			foreach (char c in xml)
			{
				if (IsLegalXmlChar(c))
				{
					buffer.Append(c);
				}
			}
		 
			return buffer.ToString();
		}

		/// <summary>
		/// Whether a given character is allowed by XML 1.0.
		/// </summary>
		public static bool IsLegalXmlChar(int character)
		{
		    return
		    (
		         character == 0x9 /* == '\t' == 9   */          ||
		         character == 0xA /* == '\n' == 10  */          ||
		         character == 0xD /* == '\r' == 13  */          ||
		        (character >= 0x20    && character <= 0xD7FF  ) ||
		        (character >= 0xE000  && character <= 0xFFFD  ) ||
		        (character >= 0x10000 && character <= 0x10FFFF)
		    );
		}


		public static string IndentedXml(XmlDocument xml)
		{
			StringBuilder sb = new StringBuilder();
			XmlWriterSettings settings = new XmlWriterSettings();
			settings.Encoding = Encoding.UTF8;
			settings.Indent = true;
			settings.NewLineChars = Environment.NewLine;
			settings.NewLineHandling = NewLineHandling.Replace;
			XmlWriter wri = XmlWriter.Create(sb, settings);
			xml.Save(wri);
			wri.Close();
			return sb.ToString();
		}
		public static string IndentedXml(XElement element)
		{
			StringBuilder sb = new StringBuilder();
			XmlWriterSettings settings = new XmlWriterSettings();
			settings.Encoding = Encoding.UTF8;
			settings.Indent = true;
			settings.NewLineChars = Environment.NewLine;
			settings.NewLineHandling = NewLineHandling.Replace;
			XmlWriter wri = XmlWriter.Create(sb, settings);
			element.WriteTo(wri);
			wri.Close();
			return sb.ToString();
		}

		public static XmlDocument TransformXml(XslCompiledTransform xslt, IXPathNavigable xml)
		{
			XmlWriterSettings resultSettings = new XmlWriterSettings();
			resultSettings.Encoding = Encoding.UTF8;
			resultSettings.CloseOutput = false;
			resultSettings.Indent = true;
			XmlDocument result;
			using (MemoryStream xmlResult = new MemoryStream())
			{
				XmlWriter resultWriter = XmlWriter.Create(xmlResult, resultSettings);

				XsltArgumentList xslOptions = new XsltArgumentList();
				xslOptions.AddParam("now", "", DateTime.Now.ToString());
				// xslOptions.AddExtensionObject( "urn:toto", toto );
				xslt.Transform(xml, xslOptions, resultWriter);

				xmlResult.Position = 0;
				result = new XmlDocument();
				result.Load(xmlResult);

				resultWriter.Close();
				xmlResult.Close();
			}
			return result;
		}
		
		public static XDocument LinqTransformXml(XslCompiledTransform xslt, IXPathNavigable xml)
		{
			XmlWriterSettings resultSettings = new XmlWriterSettings();
			resultSettings.Encoding = Encoding.UTF8;
			resultSettings.CloseOutput = false;
			resultSettings.Indent = true;

			XDocument xdoc;
			using (MemoryStream xmlResult = new MemoryStream())
			{
				XmlWriter resultWriter = XmlWriter.Create(xmlResult, resultSettings);

				XsltArgumentList xslOptions = new XsltArgumentList();
				xslOptions.AddParam("now", "", DateTime.Now.ToString());
				// xslOptions.AddExtensionObject( "urn:toto", toto );
				xslt.Transform(xml, xslOptions, resultWriter);

				xmlResult.Position = 0;
				XmlReader resultReader = XmlReader.Create(xmlResult);
				xdoc = XDocument.Load(resultReader);
				
				resultWriter.Close();
				xmlResult.Close();
			}
			return xdoc;
		}
		public static XmlDocument LoadXmlResource(string filename)
		{
			String resourceName = Assembly.GetExecutingAssembly().FullName.Split(new char[] { ',' })[0] + ".Resources." + filename;
			XmlDocument document;
			using (Stream inputStream = Assembly.GetExecutingAssembly().GetManifestResourceStream(resourceName))
			{
				if (null == inputStream)
				{
					throw new FileNotFoundException(resourceName);
				}

				inputStream.Position = 0;
				document = new XmlDocument();
				document.Load(inputStream);

				inputStream.Close();
			}

			return document;
		}

		private static string[][] TimeZones = new string[][] {
			new string[] {"ACDT", "+1030", "Australian Central Daylight"},
			new string[] {"ACST", "+0930", "Australian Central Standard"},
			new string[] {"ADT", "-0300", "(US) Atlantic Daylight"},
			new string[] {"AEDT", "+1100", "Australian East Daylight"},
			new string[] {"AEST", "+1000", "Australian East Standard"},
			new string[] {"AHDT", "-0900", ""},
			new string[] {"AHST", "-1000", ""},
			new string[] {"AST", "-0400", "(US) Atlantic Standard"},
			new string[] {"AT", "-0200", "Azores"},
			new string[] {"AWDT", "+0900", "Australian West Daylight"},
			new string[] {"AWST", "+0800", "Australian West Standard"},
			new string[] {"BAT", "+0300", "Bhagdad"},
			new string[] {"BDST", "+0200", "British Double Summer"},
			new string[] {"BET", "-1100", "Bering Standard"},
			new string[] {"BST", "-0300", "Brazil Standard"},
			new string[] {"BT", "+0300", "Baghdad"},
			new string[] {"BZT2", "-0300", "Brazil Zone 2"},
			new string[] {"CADT", "+1030", "Central Australian Daylight"},
			new string[] {"CAST", "+0930", "Central Australian Standard"},
			new string[] {"CAT", "-1000", "Central Alaska"},
			new string[] {"CCT", "+0800", "China Coast"},
			new string[] {"CDT", "-0500", "(US) Central Daylight"},
			new string[] {"CED", "+0200", "Central European Daylight"},
			new string[] {"CET", "+0100", "Central European"},
			new string[] {"CST", "-0600", "(US) Central Standard"},
			new string[] {"CENTRAL", "-0600", "(US) Central Standard"},
			new string[] {"EAST", "+1000", "Eastern Australian Standard"},
			new string[] {"EDT", "-0400", "(US) Eastern Daylight"},
			new string[] {"EED", "+0300", "Eastern European Daylight"},
			new string[] {"EET", "+0200", "Eastern Europe"},
			new string[] {"EEST", "+0300", "Eastern Europe Summer"},
			new string[] {"EST", "-0500", "(US) Eastern Standard"},
			new string[] {"EASTERN", "-0500", "(US) Eastern Standard"},
			new string[] {"FST", "+0200", "French Summer"},
			new string[] {"FWT", "+0100", "French Winter"},
			new string[] {"GMT", "-0000", "Greenwich Mean"},
			new string[] {"Etc/GMT", "-0000", "Greenwich Mean"},
			new string[] {"GST", "+1000", "Guam Standard"},
			new string[] {"HDT", "-0900", "Hawaii Daylight"},
			new string[] {"HST", "-1000", "Hawaii Standard"},
			new string[] {"IDLE", "+1200", "Internation Date Line East"},
			new string[] {"IDLW", "-1200", "Internation Date Line West"},
			new string[] {"IST", "+0530", "Indian Standard"},
			new string[] {"IT", "+0330", "Iran"},
			new string[] {"JST", "+0900", "Japan Standard"},
			new string[] {"JT", "+0700", "Java"},
			new string[] {"MDT", "-0600", "(US) Mountain Daylight"},
			new string[] {"MED", "+0200", "Middle European Daylight"},
			new string[] {"MET", "+0100", "Middle European"},
			new string[] {"MEST", "+0200", "Middle European Summer"},
			new string[] {"MEWT", "+0100", "Middle European Winter"},
			new string[] {"MST", "-0700", "(US) Mountain Standard"},
			new string[] {"MOUNTAIN", "-0700", "(US) Mountain Standard"},
			new string[] {"MT", "+0800", "Moluccas"},
			new string[] {"NDT", "-0230", "Newfoundland Daylight"},
			new string[] {"NFT", "-0330", "Newfoundland"},
			new string[] {"NT", "-1100", "Nome"},
			new string[] {"NST", "+0630", "North Sumatra"},
			new string[] {"NZ", "+1100", "New Zealand "},
			new string[] {"NZST", "+1200", "New Zealand Standard"},
			new string[] {"NZDT", "+1300", "New Zealand Daylight "},
			new string[] {"NZT", "+1200", "New Zealand"},
			new string[] {"PDT", "-0700", "(US) Pacific Daylight"},
			new string[] {"PST", "-0800", "(US) Pacific Standard"},
			new string[] {"PACIFIC", "-0800", "(US) Pacific Standard"},
			new string[] {"ROK", "+0900", "Republic of Korea"},
			new string[] {"SAD", "+1000", "South Australia Daylight"},
			new string[] {"SAST", "+0900", "South Australia Standard"},
			new string[] {"SAT", "+0900", "South Australia Standard"},
			new string[] {"SDT", "+1000", "South Australia Daylight"},
			new string[] {"SST", "+0200", "Swedish Summer"},
			new string[] {"SWT", "+0100", "Swedish Winter"},
			new string[] {"USZ3", "+0400", "USSR Zone 3"},
			new string[] {"USZ4", "+0500", "USSR Zone 4"},
			new string[] {"USZ5", "+0600", "USSR Zone 5"},
			new string[] {"USZ6", "+0700", "USSR Zone 6"},
			new string[] {"UT", "-0000", "Universal Coordinated"},
			new string[] {"UTC", "-0000", "Universal Coordinated"},
			new string[] {"UZ10", "+1100", "USSR Zone 10"},
			new string[] {"WAT", "-0100", "West Africa"},
			new string[] {"WET", "-0000", "West European"},
			new string[] {"WST", "+0800", "West Australian Standard"},
			new string[] {"YDT", "-0800", "Yukon Daylight"},
			new string[] {"YST", "-0900", "Yukon Standard"},
			new string[] {"ZP4", "+0400", "USSR Zone 3"},
			new string[] {"ZP5", "+0500", "USSR Zone 4"},
			new string[] {"ZP6", "+0600", "USSR Zone 5"}
		};

		private static Hashtable TimeZonesTable;

		static mXslTools() {
			TimeZonesTable = new Hashtable(50);
			foreach (string[] tz in TimeZones)
			{
				TimeZonesTable.Add(tz[0], tz[1]);
			}
		}

		public static bool TryParseDateTime(string strDate, out DateTime dt)
		{
			dt = new DateTime();
			List<string> dateList = new List<string>();
			dateList.Add(strDate);
			dateList.Add(TimeZoneToTimeOffset(strDate));
			dateList.Add(RemoveDayName(strDate));	// Fixing stupid bug from some rss feeds providing invalid day name in dates
			string[] dates = dateList.Distinct().ToArray();
			foreach (string date in dates)
			{
				if (_TryParseDateTime(date, out dt))
				{
					return true;
				}
			}
			ConsoleColor c = Console.ForegroundColor;
			Console.ForegroundColor = ConsoleColor.Red;
			Console.WriteLine("Invalid date: [" + strDate + "]");
			Console.ForegroundColor = c;
			return false;
		}
		public static DateTime ParseDateTime(string strDate)
		{
			List<string> dateList = new List<string>();
			dateList.Add(strDate);
			dateList.Add(TimeZoneToTimeOffset(strDate));
			dateList.Add(RemoveDayName(strDate));	// Fixing some erroneaous datetime format from rss feeds
			string[] dates = dateList.Distinct().ToArray();
			Exception e = new FormatException("Invalid date format");
			foreach (string date in dates)
			{
				try
				{
					DateTime dt = _ParseDateTime(date);
					return dt;
				}
				catch (Exception ex)
				{
					e = ex;
				}
			}
			ConsoleColor c = Console.ForegroundColor;
			Console.ForegroundColor = ConsoleColor.Red;
			Console.WriteLine("Invalid date: [" + strDate + "]");
			Console.ForegroundColor = c;
			throw e;
		}

		public static string TimeZoneToTimeOffset(string strDate)
		{
			if (0 == strDate.Length)
			{
				return string.Empty;
			}
			int TZstart = strDate.LastIndexOf(" ");
			if (TZstart != -1)
			{
				string TZstr = strDate.Substring(TZstart + 1); // +1 to avoid the space character
				string TZvalue = TimeZonesTable[TZstr] as string;
				if (TZvalue != null)
				{
					return strDate.Replace(TZstr, TZvalue);
				}
			}
			return strDate;
		}
		public static string RemoveDayName(string strDate)
		{
			if (0 == strDate.Length)
			{
				return string.Empty;
			}
			int dteStart = strDate.IndexOf(", ");
			if (dteStart != -1)
			{
				strDate = strDate.Substring(dteStart + 2, strDate.Length - (dteStart + 2));
			}
			int isx = strDate.IndexOf(" : ");
			if (-1 != isx)
			{
				string[] tmp = new string[2];
				tmp[0] = strDate.Substring(0, isx);
				tmp[1] = strDate.Substring(isx + 3);
				strDate = tmp[0] + " " + tmp[1];
			}
			return strDate;
		}
		protected static bool _TryParseDateTime(string strDate, out DateTime dt)
		{
			dt = new DateTime();
			try
			{
				dt = _ParseDateTime(strDate);
				return true;
			}
			catch (Exception)
			{
				/* Swallow */
			}
		
			return false;
		}
		protected static DateTime _ParseDateTime(string strDate)
		{
			DateTime outDate;

			// Standard Parsing
			if (true == DateTime.TryParse(strDate, out outDate)){
				DateTime dt = DateTime.Parse(strDate);
				// dt = DateTime.SpecifyKind(dt, DateTimeKind.Utc);
				if (DateTimeKind.Unspecified == dt.Kind)
				{
					dt = DateTime.SpecifyKind(dt, DateTimeKind.Utc);
				} else
				if (DateTimeKind.Local == dt.Kind)
				{
					dt = dt.ToUniversalTime();
				}
				return dt;
			}

			// Localized & Manual Parsing
			CultureInfo[] cultures = new CultureInfo[] {
				CultureInfo.InvariantCulture,
				CultureInfo.CreateSpecificCulture("en-US"),
				CultureInfo.CreateSpecificCulture("en-UK"),
				CultureInfo.CreateSpecificCulture("fr-FR"),
				CultureInfo.CreateSpecificCulture("de-DE"),
				CultureInfo.CreateSpecificCulture("es-ES"),
				CultureInfo.CreateSpecificCulture("it-IT"),
			};
			FormatException ex = new FormatException();
			foreach( CultureInfo culture in cultures ) {
				try
				{
					outDate = DateTime.ParseExact(
						strDate, 
						new string[] { 
							"R", "s", "u", "U", "f", "F", "g", "G", "D", "d",
							"dd/MM/yyyy hh:mm:ss",
							"dd/MM/yyyy hh:mm",
							"DDD, dd MMM yyy hh:mm zzz",
							"DDD, dd MMM yyy hh:mm:ss zzz",
							"DDD, dd MMM yyy hh:mm zz",
							"DDD, dd MMM yyy hh:mm:ss zz",
							"DDD, dd MMM yyy hh:mm z",
							"DDD, dd MMM yyy hh:mm:ss z",
							"DDD, dd MMM yyy hh:mm K",
							"DDD, dd MMM yyy hh:mm:ss K",	
							"dd MMM yyy hh:mm zzz",
							"dd MMM yyy hh:mm:ss zzz",
							"dd MMM yyy hh:mm zz",
							"dd MMM yyy hh:mm:ss zz",
							"dd MMM yyy hh:mm z",
							"dd MMM yyy hh:mm:ss z",
							"dd MMM yyy hh:mm K",
							"dd MMM yyy hh:mm:ss K"
						}, 
						culture, 
						DateTimeStyles.AdjustToUniversal | DateTimeStyles.AssumeUniversal
					);
					
				}
				catch (FormatException e)
				{
					ex = e;
					continue;
				}
				return outDate;
			}
			throw new FormatException("Invalid DateTime format '" + strDate + '"', ex);
		}
	}
}
