<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
 */

namespace MetaNull\Application {

	/**
	 * Base class defining the mechanism of an application
	 * @author Pascal Havelange
	 * @copyright © EUROPEAN COMMISSION, 2008
	 * @package MetaNull\Application
	 * @license http://ec.europa.eu/idabc/eupl
	 */
	abstract class mApplication extends mBaseApplication {

		/**
		 * Parsed configuration file
		 * @var array
		 */
		protected $config;

		/**
		 * Save configuration on exit
		 * @var bool
		 */
		protected $auto_save_config = false;

		/**
		 * Full path to the file where to save the configuration on exit
		 * @var string
		 */
		protected $auto_save_config_path = false;

		/**
		 * Format of the file where to save the configuration on exit
		 * @var string
		 */
		protected $auto_save_config_format = 'xml';

		/**
		 * Object Constructor
		 * @param array|string  $application_config An associative array containing the configuration OR the path to a configuration file
		 * @param string $config_format Optional, (Ignored/invalid if the first argument is not a string), the format of the configuration file (can be 'xml', 'inf', ...)
		 */
		public function __construct($application_config) {
			$argc = count($application_config);
			parent::__construct();
			if ($argc == 2 && is_string($application_config['path']) && is_string($application_config['format'])){
				$this->config = \MetaNull\Tools\mConfig::loadConfiguration($application_config['path'], $application_config['format'], 'app_config');
				$this->auto_save_config_path = $application_config['path'];
				$this->auto_save_config_format = $application_config['format'];
				$this->auto_save_config = false;
			}
			else if ($argc == 1 && is_array($application_config['config'])){
				$this->config = $application_config['config'];
				$this->auto_save_config_path = null;
				$this->auto_save_config_format = 'xml';
				$this->auto_save_config = false;
			} else {
				throw new \InvalidArgumentException('mApplication::__construct(...): Argument should be a valid path or an associative array containing the configuration tree');
			}

			if( is_array( $this->config ) && array_key_exists('datetime', $this->config )) {
				if( is_array( $this->config['datetime'] ) && array_key_exists('timezone', $this->config['datetime'] )) {
					$tz = new \DateTimeZone( $this->config['datetime']['timezone'] );
					$tm = new \DateTime( 'now', $tz );
					date_default_timezone_set( $tz->getName() );
				}
			}
		}

		/**
		 * Returns the multidimentionnal array containing the configuration of the application
		 * @return array
		 */
		final public function getConfig() {
			return $this->config;
		}

		/**
		 * This method must be implemented in the derivated class
		 * It is called first (prior to the onExecute() method)
		 * Throw an Exception from this method to immediatelly stop the application (in which case onDestroy() will be gracefuly called)
		 * @access protected
		 */
		protected function onInitialize() {

		}

		/**
		 * This method must be implemented in the derivated class
		 * This method is called last (after the onExecute() method)
		 * No exception should be thrown within this method, its only goal is to possibly free some resources
		 * @access protected
		 */
		protected function onDestroy() {
			if( $this->auto_save_config ) {
				$this->saveConfiguration( $this->auto_save_config_path, $this->auto_save_config_format );
			}
			parent::onDestroy();
		}

		/**
		 * Set the auto-save option for the configuration file
		 * @param bool $auto_save If set to true, the configuration file will be saved when the application exists
		 */
		final public function setAutoSaveConfig( $auto_save ) {
			$this->auto_save_config = $auto_save;
		}

	}

}
?>