<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
 */

namespace MetaNull\Commission {

	/**
	 * EC_User stores the information about an user of the application
	 * iUser implementation, focussed on the usage of the ECAS system
	 * The commission's LDAP database is used to retrieve user attributes
	 * @author Pascal Havelange
	 * @copyright © EUROPEAN COMMISSION, 2008
	 * @package MetaNull\Commission
	 * @license http://ec.europa.eu/idabc/eupl
	 */
	class EC_User implements \MetaNull\Authentication\iUser {

		/**
		 * ECAS User ID
		 * @var string
		 */
		public $user_id = false;

		/**
		 * LDAP Attributes
		 * @var LDAPEntry
		 */
		public $ldap_attributes = null;

		/**
		 * The user preferences
		 * @var array
		 * @access public
			*/
		protected $user_preferences = array();

		/**
		 * Constructor
		 * @param string $user_id UserID
		 * @param bool $fetch_ldap If set to true, tries to retrieve LDAP Attributes for the user
		 */
		public function __construct( $user_id, $fetch_ldap = true ) {
			$this->user_id = $user_id;
			$this->ldap_attributes = array();
			if( $fetch_ldap ) {
				$ldap_query = "(&(objectClass=person)(uid={$user_id}))";
				$this->ldap_attributes = EC_User::fetchLDAP( $user_id );
			}
		}

		/**
		 * Returns the UserId of the user
		 * @return string
		 */
		public function getUserId() {
			return $this->user_id;
		}

		/**
		 * Returns an array of valid attributes associated with
		 * @return array
		 */
		public function getAttributeList() {
			return array_keys( EC_LDAP::$ldap_attributes );
		}

		/**
		 * Check the group membership of an user
		 * @param string|array $groups Either the name of a group or an array of group names against which to check the membership of the user
		 * @param bool $binary_and If set to true, the user must be a member of ALL the groups in the list, otherwise it must belong to at least one of the groups
		 * @return bool
		 */
		public function isMemberOf( $groups, $binary_and = false ) {
			if( 0 == count( $this->ldap_attributes ) || false === $this->user_id  ) {
				throw new \MetaNull\Authentication\authenticationException( 'Not authenticated/No LDAP attributes available' );
			}
			if( !is_array( $groups )) {
				return \MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $groups );
			} else {
				if( false === $binary_and ) {
					foreach( $groups as $group ) {
						if( \MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $group )) {
							return true;
						}
					}
					return false;
				} else {
					foreach( $groups as $group ) {
						if( !\MetaNull\Net\mLDAP::testAttributeI( $this->ldap_attributes, 'cudgroup', $group )) {
							return false;
						}
					}
					return true;
				}
			}
		}

		/**
		 * Accessor, quickly access the user's preferences
		 * @return string
		 */
		public function __get( $attributeName ) {
			if( $this->__isset( $attributeName ) ) {
				return $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]];
			} else {
				return false;
				//throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
			}
		}

		/**
		 * Accessor, quickly access the user's preferences
		 */
		public function __set( $attributeName, $value ) {
			if( $this->__isset( $attributeName ) ) {
				$this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] = $value;
			} else {
				//throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
			}
		}

		/**
		 * Accessor, quickly access the user's attributes using object style syntax
		 * @return bool
		 */
		public function __isset( $attributeName ) {
			if( 0 == count( $this->ldap_attributes ) ||
			!isset(EC_LDAP::$ldap_attributes[$attributeName]) ||
			!isset( $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] )
			) {
				return false;
			}
			return true;
		}

		/**
		 * Accessor, quickly access the user's attributes using object style syntax
		 * @return bool
		 */
		public function __unset( $attributeName ) {
			if( $this->__isset( $attributeName ) ) {
				unset( $this->ldap_attributes[EC_LDAP::$ldap_attributes[$attributeName]] );
			} else {
				throw new \MetaNull\Authentication\authenticationException( "LDAP Attribute '{$attributeName}' not found!" );
			}
		}

		/**
		 * Get user attributes from the LDAP database
		 *
		 * @param string $user_id Id of the user in the LDAP database
		 * @return array An array of attributes
		 * @static
		 * @access public
		 */
		static public function fetchLDAP( $user_id ) {
			$ldap = new EC_LDAP();
			if( false === $ldap->connect() ) {
				throw new \MetaNull\Authentication\authenticationException( "Can't connect to the LDAP Server!" );
			}
			$results = $ldap->search( "(&(objectClass=person)(uid={$user_id}))" );
			if( 0 == count( $results ) || !isset( $results[0] )) {
				throw new \MetaNull\Authentication\authenticationException( "User '{$user_id}' not found in the LDAP database!" );
			}
			return $results[0];
		}
	}

}
?>