<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3{

/**
 * Base class for all modules of the application
 * @package tariqa3
 * @author Pascal Havelange, Cosmin Popescu
 * @copyright © EUROPEAN COMMISSION, 2009-2010
 * @license http://ec.europa.eu/idabc/eupl
 */
abstract class BaseModule extends \MetaNull\Application\mModule {

    /**
     * Informations returned by the getInfo() method
     * It can contains the title and description of the module, the http-headers, and specific html informations to put in the page header
     * @var $infos
     * @access protected
     */
    protected $infos = array();

    /**
     * The list of blacklisted infos. If an info is on this list, it will not be processed
     * @var array
     * @access protected
     */
    protected $blacklisted_infos = array();

    /**
     * Name of the subfolders where the files of the current module are stored (relative to the module_path and template_path defined in the configuration file)
     * @var string
     * @access protected
     **/
    protected $base_folder = '';

    /**
     * If true, prevent browsers from caching javascript or CSS files
     * @var boolean
     * @access protected
     */
    protected $prevent_cache = false;

    /**
     * Build the module and parse the config file
     * @param array $config The multidimentionnal config array
     * @param string $charset The characterset to use in the webpage
     */
    public function __construct( $config, $charset='UTF-8' ) {
	parent::__construct( $config );
	if (array_key_exists('prevent-cache', $this->config['application'])){
	    if ($this->config['application']['prevent-cache'] == 1){
		$this->prevent_cache = true;
	    }
	}
	$this->infos = array(
		'charset' => $charset
	);
    }

    protected function onInitialize() {
	parent::onInitialize();
    }

    /**
     * Returns an associative array of strings giving information about the module
     * @return array
     */
    public function getInfo() {
	if( 0 == count($this->infos )) {
	    return parent::getInfo();
	}
	return $this->infos;
    }

    /**
     * Set information for the template: Title of the webpage
     * @param string $value Title of the webpage
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setTitle( $value, $ignore_blacklist = false ) {
	if ($ignore_blacklist || !array_key_exists('title', $this->blacklisted_infos)){
	    $this->infos['title'] = htmlspecialchars( $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: Description of the webpage
     * @param string $value Description of the webpage
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setDescription( $value, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('description', $this->blacklisted_infos)){
	    $this->infos['description'] = htmlspecialchars( $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: Content-Type and charset of the webpage
     * @param string $type Content-Type of the webpage
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setContentType( $type, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('content-type', $this->blacklisted_infos)){
	    $this->infos['content-type'] = htmlspecialchars( $type, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: Author of the webpage
     * @param string $value Author of the webpage
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setAuthor( $value, $ignore_blacklist = false ) {
	if ($ignore_blacklist || !array_key_exists('author', $this->blacklisted_infos)){
	    $this->infos['author'] = htmlspecialchars( $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: Lastu-pdated date of the webpage
     * @param string $value Last-updated date of the webpage. Default: current time
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setLastUpdated( $value = false, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('last-updated', $this->blacklisted_infos)){
	    $this->infos['last-updated'] = htmlspecialchars( false === $value ? date( 'r' ) : $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: Copyright of the webpage
     * @param string $value Copyright of the webpage
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setCopyright( $value, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('copyright', $this->blacklisted_infos)){
	    $this->infos['copyright'] = htmlspecialchars( $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Set information for the template: onload javascript code for HTML/BODY
     * @param string $value onload javascript code for HTML/BODY
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setBodyOnLoad( $value, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('onload', $this->blacklisted_infos)){
	    $this->infos['onload'] = (array_key_exists('onload', $this->infos)?$this->infos['onload'] . ';':'') . htmlspecialchars( $value, ENT_COMPAT, $this->infos['charset'] );
	}
    }

    /**
     * Sets the raw headers (free text to be added in the <head> section of the document before any other header)
     * @param string $value The headers
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @access protected
     * @final
     * @return void
     */
    final protected function info_setRawHeaders($value, $ignore_blacklist = false ) {
	if ($ignore_blacklist || !array_key_exists('raw_headers', $this->blacklisted_infos)){
	    $this->infos['raw_headers'] = (array_key_exists('raw_headers', $this->infos)?$this->infos['raw_headers']:'') . $value;
	}
    }

    /**
     * Set information for the template: onunload javascript code for HTML/BODY
     * @param string $value onunload javascript code for HTML/BODY
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setBodyOnUnload( $value, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('onunload', $this->blacklisted_infos)){
	    $this->infos['onunload'] = htmlspecialchars( $value, ENT_QUOTES, $this->infos['charset'] );
	}
    }

    /**
     * Prevents the browser cache
     * @param string $file The file name
     * @access private
     * @return string
     */
    private function preventCache($file) {
	if (!$this->prevent_cache){
	    return $file;
	}
	return $file . (strpos($file, '?') === false?'?':'&') . 'rnd=' . rand();
    }

    /**
     * Set information for the template: List of external stylesheets to use
     * @param mixed $path The path to a css file, or an array of pathes to css files.
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setStyleSheet( $path, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('stylesheets', $this->blacklisted_infos)){
	    if (!array_key_exists('stylesheets', $this->infos)) $this->infos['stylesheets'] = array();
	    if( is_array( $path )) {
		foreach( $path as $p ) {
		    $this->infos['stylesheets'][] = array( 'path' => htmlspecialchars( $this->preventCache($p), ENT_COMPAT, $this->infos['charset'] ) );
		}
	    } else {
		$this->infos['stylesheets'][] = array( 'path' => htmlspecialchars( $this->preventCache($file), ENT_COMPAT, $this->infos['charset'] ) );
	    }
	}
    }

    /**
     * Set information for the template: List of external javascript sources to use
     * @param mixed $path The path to a javascript file, or an array of pathes to javascript files.
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setJavascript( $path, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('javascript', $this->blacklisted_infos)){
	    if (!array_key_exists('javascript', $this->infos)) $this->infos['javascript'] = array();
	    if( is_array( $path )) {
		foreach( $path as $p ) {
		    $this->infos['javascript'][] = array( 'path' => htmlspecialchars( $this->preventCache($p), ENT_NOQUOTES, $this->infos['charset'] ) );
		}
	    } else {
		$this->infos['javascript'][] = array( 'path' => htmlspecialchars( $this->preventCache($path), ENT_NOQUOTES, $this->infos['charset'] ) );
	    }
	}
    }

    /**
     * Set information for the template: List of inline stylesheets to use
     * @param mixed $css A block of css, or an array of blocks of css.
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setInlineStylesheet( $css, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('stylesheets/inline', $this->blacklisted_infos)){
	    $this->infos['stylesheets/inline'] = array();
	    if( is_array( $css )) {
		foreach( $css as $p ) {
		    $this->infos['stylesheets/inline'][] = array( 'css' => /*htmlspecialchars(*/ $p, /*ENT_COMPAT, $this->infos['charset'] )*/ );
		}
	    } else {
		$this->infos['stylesheets/inline'][] = array( 'css' => /*htmlspecialchars(*/ $css, /*ENT_COMPAT, $this->infos['charset'] )*/ );
	    }
	}
    }

    /**
     * Set information for the template: List of inline javascript code to use
     * @param mixed $js A block of javscript code, or an array of block of javascript code.
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setInlineJavascript( $js, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists('javascript/inline', $this->blacklisted_infos)){
	    if (!array_key_exists('javascript/inline', $this->infos)) $this->infos['javascript/inline'] = array();
	    if( is_array( $js )) {
		foreach( $js as $p ) {
		    $this->infos['javascript/inline'][] = array( 'script' => /*htmlspecialchars(*/ $p /*, ENT_NOQUOTES, $this->infos['charset'] )*/ );
		}
	    } else {
		$this->infos['javascript/inline'][] = array( 'script' => /*htmlspecialchars(*/ $js /*, ENT_NOQUOTES, $this->infos['charset'] )*/ );
	    }
	}
    }

    /**
     * Set information for the tempate: Custom information
     * @param string $key The key of the template value
     * @param mixed $data The value
     * @param boolean $ignore_blacklist If true, then ignore the blacklisted infos [OPTIONAL]
     * @final
     * @access protected
     */
    final protected function info_setCustom( $key, $data, $ignore_blacklist = false  ) {
	if ($ignore_blacklist || !array_key_exists($key, $this->blacklisted_infos)){
	    $this->infos[$key] = $data;
	}
    }

    /**
     * The logoff module do not update the configuration
     */
    public function hasUpdatedConfiguration() {
	return false;
    }

    /**
     * Returns the Full path to the templates folder of this application (no trailing slash)
     * @access public
     * @return string
     */
    public final function getTemplatesFolder() {
	return "{$this->config['templates']['base']}/{$this->config['templates']['default']}/{$this->base_folder}";
    }
    /**
     * Returns the full path to a specific template file of the application
     * @param string $template_file Template file's name
     * @return string
     * @access public
     */
    public final function getTemplateFilePath( $template_file ) {
	return $this->getTemplatesFolder() . '/' . $template_file;
    }

    /**
     * Returns the base url to the templates folder of this application's specific module (no trailing slash)
     * @access public
     * @return string
     */
    public final function getTemplatesUrl() {
	return "{$this->config['templates']['baseurl']}/{$this->config['templates']['default']}/{$this->base_folder}";
    }
    /**
     * Returns the full url to a specific template file of the application's specific module
     * @param string $template_file Template file's name
     * @return string
     * @access public
     */
    public final function getTemplateFileUrl( $template_file ) {
	return $this->getTemplatesUrl() . '/' . $template_file;
    }

    /**
     * Returns the base url to the templates folder of the application (no trailing slash)
     * @access public
     * @return string
     */
    public final function getThemeUrl() {
	return "{$this->config['templates']['baseurl']}/{$this->config['templates']['default']}";
    }
    /**
     * Returns the full url to a specific template file of the application
     * @param string $template_file Template file's name
     * @return string
     * @access public
     */
    public final function getThemeFileUrl( $template_file ) {
	return $this->getThemeUrl() . '/' . $template_file;
    }

    /**
     * Returns the base url to the templates folder of the application (no trailing slash)
     * @access public
     * @return string
     */
    public final function getThemePath() {
	return "{$this->config['templates']['base']}/{$this->config['templates']['default']}";
    }
    /**
     * Returns the full url to a specific template file of the application
     * @param string $template_file Template file's name
     * @return string
     * @access public
     */
    public final function getThemeFilePath( $template_file ) {
	return $this->getThemePath() . '/' . $template_file;
    }

    /**
     * Returns the Full path to the modules folder of this application (no trailing slash)
     * @access public
     * @return string
     */
    public final function getModuleFolder() {
	return "{$this->config['modules']['base']}/{$this->base_folder}";
    }
    /**
     * Returns the base url to the modules folder of this application (no trailing slash)
     * @access public
     * @return string
     */
    public final function getModuleUrl() {
	return "{$this->config['modules']['baseurl']}/{$this->base_folder}";
    }

}

}
?>