<?php
/**
 * Class used for rendering the list of themes
 *
 * @license http://ec.europa.eu/idabc/eupl
 */
namespace tariqa3{

class ThemeTextsSection extends tariqa3ModuleSection{

	/**
	 * The theme for which we are displaying the topics
	 * @var Theme
	 * @access private
	 */
	private $theme;

	/**
	 * The translation array
	 * @var array
	 * @access public
	 */
	public $translationTable;
	
	/**
	 * The list of users that can edit this theme
	 * @var ThemeUsers
	 * @access private
	 */
	private $users;

	/**
	 * Constructs the object
	 * @param array $config The configuration array
	 * @param string $parentBase base_folder of the container class
	 * @param string $section_id The section identifier
	 * @param Theme $theme The theme for which we are displaying the topics
	 * @param array $translationTable The translation table
	 * @access public
	 * @return void
	 */
	public function __construct($config, $parentBase, $section_id, $theme, $translationTable) {
		parent::__construct($config, $parentBase, $section_id);
		$this->translationTable = $translationTable;

		$this->theme = $theme;
		$this->users = statics::ListThemeUsers($this->theme->theme_id);
	}

	/**
	 * Saves the long text and the short text of a theme
	 * @param tariqaUser $user The user which is trying to save the texts
	 * @param int $theme_id The theme identifier
	 * @param string $short_text The short text
	 * @param string $long_text The long text
	 * @access public
	 * @static
	 * @return string
	 */
	public static function saveTexts($user, $theme_id, $short_text, $long_text) {
		if (!Themes::canEditTheme($user, $theme_id)){
			return "You cannot edit this theme";
		}
		try{
			statics::$tariqaClient->SetThemeText(statics::$tariqaToken, $theme_id, $short_text, $long_text);
		}
		catch (\Exception $e){
			error_log(\MetaNull\Tools\mDump::dumpVariable($e, true));
			return 'There was an error saving the texts. The error message is ' . $e->getMessage();
		}

		return 'Ok';
	}

	/**
	 * Render the module
	 * @return string
	 */
	public function getPageContent() {
		$tpl = new \MetaNull\Application\MTemplate( 'm' );
		$tpl->loadTemplate($this->getTemplateFilePath('texts.html'));
		$tpl->data = $this->translationTable;

		$tpl->data['img_path'] = $this->config['templates']['baseurl'] . '/' . $this->config['templates']['default'] . '/images';
		$tpl->data['short_text'] = $this->theme->short_text;
		$tpl->data['long_text'] = $this->theme->long_text;
		$tpl->data['theme_id'] = $this->theme->theme_id;
		$tpl->data['title'] = $this->theme->title;

		$tpl->data['can_edit'] = false;
		$tpl->data['editing_form'] = '';
		if (Themes::canEditTheme($this->user, $this->theme->theme_id)){
			$tpl->data['can_edit'] = true;
			$tpl2 = new \MetaNull\Application\MTemplate('m');
			$tpl2->loadTemplate($this->getTemplateFilePath('edit_form.html'));
			$tpl2->data = $this->translationTable;
			$tpl2->data['title'] = $this->theme->title;
			$tpl2->data['short_text'] = $this->theme->short_text;
			$tpl2->data['long_text'] = $this->theme->long_text;

			$tpl->data['editing_form'] = $tpl2->transform();
		}

		$this->info_setJavascript(
			array(
				'scripts/theme_texts.js',
				'scripts/wysiwyg.js',
				'scripts/wysiwyg-settings.js',
			)
		);
		$this->info_setStyleSheet(array(
			$this->config['templates']['baseurl'] . '/' . $this->config['templates']['default'] . '/wysiwyg/styles/editor_styles.css', 
		));

		if (in_array($this->user->user_id, $this->config['security']['debug-users']['user'])){
			$js_code = <<<js
small.Toolbar[0].push('viewSource');
	
js;
			$this->info_setInlineJavascript($js_code);
		}

		$this->info_setBodyOnLoad('setJQueryUITopics()');

		return $tpl->transform();
	}
}
}
?>
