<?php
/**
 * Class used for rendering the list of themes
 *
 * @license http://ec.europa.eu/idabc/eupl
 */
namespace tariqa3{

require_once('FilesTree.class.php');

class UploadSection extends tariqa3ModuleSection{
	
	/**
	 * The theme page type (used to identify the page in which the control will be displayed)
	 * If you want to display the control into another page, add the type here and add the folder into the config file
	 * @var string
	 * @access public
	 */
	const THEME_PAGE = 'theme';

	/**
	 * The page in which the control is displayed
	 * @var string
	 * @access private
	 */
	private $page;

	/**
	 * The root folder of the uploads
	 * @var string
	 * @access private
	 */
	private $root_folder = '';

	/**
	 * The folder path
	 * @var string
	 * @access private
	 */
	private $upload_path;

	/**
	 * The tree HTML identifier
	 * @var string
	 * @access protected
	 */
	protected $tree_id = null;

	/**
	 * The title of the documents repository
	 * @var string
	 * @access private
	 */
	private $obj_title = null;

	/**
	 * Constructs the object
	 * @param array $config The configuration array
	 * @param string $parentBase base_folder of the container class
	 * @param string $section_id The section identifier
	 * @param array $translationTable The translation table
	 * @param string $page The page in which the control will be displayed
	 * @param string $tree_id The tree HTML identifier
	 * @param string $root_folder The root folder of the uploads [OPTIONAL]
	 * @param string $title The title of the document repository
	 * @access public
	 * @return void
	 */
	public function __construct($config, $parentBase, $section_id, $translationTable, $page, $tree_id, $root_folder = '', $title = null) {
		parent::__construct($config, $parentBase, $section_id);
		$this->translationTable = $translationTable;
		$this->page = $page;
		$this->root_folder = $root_folder;

		$this->upload_path = self::getFolder($this->page, $this->root_folder);
		$this->tree_id = $tree_id;
		$this->obj_title = $title;
	}

	/**
	 * If true, then the user can modify the current folder
	 * @param tariqaUser $user The current user
	 * @param string $folder The folder
	 * @param string $config The application configuration array
	 * @access public
	 * @static
	 * @return boolean
	 */
	public static function canUpload($user, $folder, $config) {
	    $folder = str_replace("\\", '/', $folder);
	    $_folders = $config['uploader']['folders']['folder'];
	    if (!isset($_folders[0])){
		$_folders = array($_folders);
	    }
	    foreach ($_folders as $f){
		$path = strtolower(str_replace(array('/', '.', ':'), array("\\/", "\\.", "\\:"), $f['path']));
		if (preg_match('/^' . $path . '/', strtolower($folder))){
		    if (array_key_exists($user->user_id, $f)){
			return true;
		    }
		    else {
			return false;
		    }
		}
	    }

	    return false;
	}

	/**
	 * Retrieves the folder in which the upload will take place
	 * @param string $page The page in which the control is displayed
	 * @param string $root_folder The root folder of the upload [OPTIONAL]
	 * @access public
	 * @static
	 * @return string
	 */
	public static function getFolder($page, $root_folder = '') {
		$config = \MetaNull\Tools\mConfig::loadConfiguration(APPLICATION_MAIN_CONFIG_FILE);
		$folder = $config['uploader'][$page];
		$folder = str_replace("\\", '/', $folder);
		if (substr($folder, -1) != '/') $folder .= '/';
		$folder .= $root_folder;
		$folder = str_replace("\\", '/', $folder);
		if (substr($folder, -1) != '/') $folder .= '/';

		if (!is_dir($folder)){
			if (mkdir($folder, 0755) === false){
				throw new \Exception('Unable to read the upload folder');
			}
		}

		return $folder;
	}

	/**
	 * Moves a folder from source to destination
	 * @param string $source The source path
	 * @param string $destination The destination path
	 * @access public
	 * @static
	 * @return void
	 */
	public static function m_rename($source, $destination) {
		if (!rename($source, $destination)){
			if (is_dir($destination)){
				$dh = opendir($source);
				while ($file = readdir($dh)) {
					if ($file != "." && $file != "..") {
						if (is_dir($source . $file)) {
							self::m_rename($source . $file . '/', $destination . $file . '/');
						}
						else {
							if (file_exists($destination . $file)) {
								unlink($destination . $file);
							}
							rename($source . $file, $destination . $file);
						}
					}
				}

				closedir($dh);

				rmdir($source);
			}
		}
	}

	/**
	 * Removes a file or a folder
	 * @param string $path The path to the item
	 * @access public
	 * @static
	 * @return int
	 */
	public static function m_unlink($path) {
	    if (substr($path, -1) == '/') $path = substr($path, 0, strlen($path) - 1);
		if( is_file($path) ) {
			if( is_writable($path) ) {
				if( @unlink($path) ) {
					return 1;
				}
			}
			return false;
		} else if( is_dir($path) ) {
			if( is_writeable($path) ) {
				foreach( new \DirectoryIterator($path) as $file ) {
					if( $file->isDot() ) {
						unset($file);
						continue;
					}
					$rc = false;
					if( $file->isFile() ) {
						$rc = self::m_unlink( $file->getPathName() );
					} else if( $file->isDir() ) {
						$rc = self::m_unlink( $file->getRealPath() );
					}
					unset($file);
					if( false === $rc ) {
						return false;
					} else {
						$deleted += $rc;
					}
				}
				if( @rmdir($path) ) {
					return $deleted + 1;
				} else {
					return false;
				}
			} else {
				return false;
			}
		}
		return false;
	}

	/**
	 * Removes a file or a folder
	 * @param tariqaUser $user The user trying to perform the removal
	 * @param string $page The page in which the controller is being displayed
	 * @param string $root_folder The root folder
	 * @param string $item_path The path to the item to be deleted
	 * @param array $translationTable The translation table
	 * @param array $config The application configuration array
	 * @access public
	 * @static
	 * @return string
	 */
	public static function removeItem($user, $page, $root_folder, $item_path, $translationTable, $config) {
		if (!self::canUpload($user, self::getFolder($page), $config)){
			throw new \Exception('Access denied for user ' . $user->user_id . ' to page ' . $page);
		}

                $item_path = \MetaNull\Tools\mTools::rawurldecode($item_path);

		$upload_path = self::getFolder($page) . $root_folder;
		$upload_path = str_replace("\\", '/', $upload_path);
		if (substr($upload_path, -1) != '/') $upload_path .= '/';

                $item_name = '';
                $parts = explode('/', $item_path);
                foreach ($parts as $part){
                    if ($part != ''){
                        $item_name = $part;
                    }
                }

		$path = $upload_path . $item_path;

		if (self::m_unlink($path) === false){
			throw new \Exception("The item could not be deleted: {$path}");
			return null;
		}

		return "Ok";
	}

	/**
	 * Creates a new folder
	 * @param tariqaUser $user The user which is trying to create a new folder
	 * @param string $page The current page in which the upload control is displayed
	 * @param string $root_folder The root folder (for example the theme id or the country id)
	 * @param string $name The name of the new folder
	 * @param array $translationTable The translation table
	 * @param string $config The application configuration array
	 * @access public
	 * @static
	 * @return string
	 */
	public static function createFolder($user, $page, $root_folder, $name, $translationTable, $config) {
		if (!self::canUpload($user, self::getFolder($page), $config)){
			throw new \Exception('Access denied for user ' . $user->user_id . ' to page ' . $page);
		}

		$upload_path = self::getFolder($page) . $root_folder;
		$upload_path = str_replace("\\", '/', $upload_path);
		if (substr($upload_path, -1) != '/') $upload_path .= '/';

		if (mkdir($upload_path . $name, 0755) === false){
			return "Error creating the new folder";
		}

		return "Ok";
	}

	/**
	 * Moves an item
	 * @param tariqaUser $user The current user
	 * @param string $page The current page in which the upload control is displayed
	 * @param string $root_folder The root folder (for example the theme id or the country id)
	 * @param string $type The type of the source (file or folder)
	 * @param string $source The source
	 * @param string $destination The destination
	 * @param array $translationTable The translation table
	 * @param string $config The application configuration array
	 * @access public
	 * @static
	 * @return string
	 */
	public static function moveItem($user, $page, $root_folder, $type, $source, $destination, $translationTable, $config) {
		if (!self::canUpload($user, self::getFolder($page), $config)){
			throw new \Exception('Access denied for user ' . $user->user_id . ' to page ' . $page);
		}

                $source = \MetaNull\Tools\mTools::rawurldecode($source);
                $destination = \MetaNull\Tools\mTools::rawurldecode($destination);

		$upload_path = self::getFolder($page) . $root_folder;
		$upload_path = str_replace("\\", '/', $upload_path);
		if (substr($upload_path, -1) != '/') $upload_path .= '/';
		$destination = $_GET['destination'];
		if ($destination == 'null') $destination = '';
		//$source = $_GET['source'];

		if (substr($destination, -1) != '/') $destination .= '/';

                $item_name = '';
                $parts = explode('/', $source);
                foreach ($parts as $part){
                    if ($part != ''){
                        $item_name = $part;
                    }
                }

		if ($type == 'file'){
			if (file_exists($upload_path . $destination . $item_name)) {
				unlink($root_folder . $destination . $file_name);
			}
	    	if (!rename($upload_path . $source, $upload_path . $destination . $item_name)){
	    		return $translationTable['upload/move_error'] . "\nSource: {$source}\nDestination: {$destination}";
	    	}
		}
		else {
			$destination .= $item_name . '/';
			if (substr($source, -1) != '/'){
				$source .= '/';
			}
			self::m_rename($upload_path . $source, $upload_path . $destination);
		}

		return "Ok";

	}

	/**
	 * Render the module
	 * @return string
	 */
	public function getPageContent() {
		$tpl = new \MetaNull\Application\MTemplate( 'm' );
		$tpl->loadTemplate($this->getTemplateFilePath('upload.html'));
		$tpl->data = $this->translationTable;

		$can_upload = self::canUpload($this->user, $this->upload_path, $this->config);

		$tree = new FilesTree($this->upload_path, $this->translate('recycle_bin'), $this->tree_id, $this->root_folder, $can_upload);
		$tpl->data['files_tree'] = $tree->folderHTML('themes_files_tree', null);
		$tpl->data['root_folder'] = $this->root_folder;
		$tpl->data['page_type'] = $this->page;
		$tpl->data['can_upload'] = $can_upload;
		$tpl->data['tree_id'] = $this->tree_id;
		if ($this->obj_title != null){
		    $tpl->data['doc_rep_title'] = $this->obj_title;
		}

		$this->info_setJavascript(
			array(
				'scripts/jquery.cookie.js',
				'scripts/uploader.js',
				'scripts/jquery.jstree.js',
				'scripts/ajaxfileupload.js',
			)
		);

		$dnd = ($can_upload?'true':'false');

		$this->info_setBodyOnLoad('initJQueryUploaderUI(\'' . $this->tree_id . '\', ' . $dnd . ')');

		return $tpl->transform();
	}
}
}
?>
