<?php
/*
 * © EUROPEAN COMMISSION, 2007-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs:
 *	Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *	Cosmin Popescu,   Adrian.POPESCU@ext.ec.europa.eu
*/

namespace tariqa3{

/**
 * Helper class extending the ECAS user class in order to also include the Tariqa user preferences
 * @author Cosmin Popescu
 * @copyright © EUROPEAN COMMISSION, 2010
 * @package tariqa3
 * @license http://ec.europa.eu/idabc/eupl
 */
class tariqaUser extends \MetaNull\Commission\EC_User {

    /**
     * The RELEX NG access
     * @var string

     */
    const ACCESS_NG = 'NG';

    /**
     * The RELEX MS access
     * @var string

     */
    const ACCESS_MS = 'MS';

    /**
     * The RELEX EC access
     * @var string

     */
    const ACCESS_EC = 'EC';

    /**
     * The RELEX RX access
     * @var string

     */
    const ACCESS_RX = 'RX';

    /**
     * The user preferences
     * @var array
     * @access protected
     */
    protected $user_preferences = array();

    /**
     * The list of engines to which the user has access
     * @var string[]
     * @access private
     */
    private $engines;

    /**
     * The access level of the user
     * @var string
     * @access private
     */
    private $access_level = 'NG';

    /**
     * Constructor
     * @param string $user_id UserID
     * @param bool $fetch_ldap If set to true, tries to retrieve LDAP Attributes for the user
     */
    public function __construct( $user_id, $fetch_ldap = true ) {
	parent::__construct($user_id, $fetch_ldap);
	$this->engines = $this->loadEngines();

	if ($this->isMemberOf('RELEX_TARIQARX', false)) {
	    $this->access_level = 'RX';
	}
	else if ($this->isMemberOf('EEAS_INTRANET', false)) {
	    $this->access_level = 'RX';
	}
	else if ($this->isMemberOf('RELEX_TARIQAEC', false)) {
	    $this->access_level = 'EC';
	}
	else if ($this->isMemberOf('RELEX_TARIQAEM', false)) {
	    $this->access_level = 'MS';
	}
    }

    /**
     * Gets an user preference. Returns null if the preference is not set
     * @param string $name The name of the preference
     * @access public
     * @return mixed
     */
    public function get_preference($name) {
	if (array_key_exists($name, $this->user_preferences)) {
	    $value = $this->user_preferences[$name];
	    if ($value == 'true'){
		$value = true;
	    }
	    else if ($value == 'false'){
		$value = false;
	    }
	    return $value;
	}

	return null;
    }

    /**
     * Loads the engines to which the user has access
     * @access private
     * @return string[]
     */
    private function loadEngines() {
	return array('Factiva' => true, 'Google' => true, 'GoogleBlogs' => true, 'GoogleImages' => true, 'GoogleNews' => true,
		'GoogleVideo' => true, 'GoogleWeb' => true, 'LatestNews' => true, 'LatestNews_aggregator' => true, 'LexisNexis' => true,
		'OxfordAnalytica' => true, 'RSS' => true, 'RSS_aggregator' => true, 'Website' => true, 'Virage' => true);
    }

    /**
     * Returns true if the user has access to the specified engine
     * @param string $engine_id The engine identifier
     * @access public
     * @return boolean
     */
    public function hasAccess($engine_id) {
	return array_key_exists($engine_id, $this->engines);
    }

    /**
     * Gets the user access level
     * @access public
     * @return int
     */
    public function get_access_level() {
	return $this->access_level;
    }

    /**
     * Sets an user preference.
     * To delete a preference from the database, just set the null value for it and set the $db to an instance of mMysql.
     * If you want to delete an array from the database, just put a "%" instead of the index value. For example,
     * to delete the "general/countries" array: $user->set_preference("general/countries/%", null);
     * @param string $name The preference
     * @param mixed $value The value
     * @param mMysql $db A database connection (used for saving the preference to the database) [OPTIONAL]
     * @access public
     * @return void
     */
    public function set_preference($name, $value, $db = null) {
	if ($value === true) {
	    $value = 'true';
	}
	else if ($value === 'false'){
	    $value = false;
	}
	if ($db != null) {
	    $_name = $db->real_escape_string($name);
	    if ($value == null) {
		if (substr($name, -1) == "%") {
		    $op = "like";
		}
		else {
		    $op = "=";
		}
		$sql = <<<sql
delete from `user_preferences`
where `key` {$op} '{$_name}' and `user_id` = '{$this->user_id}'
sql;

		$db->mQuery($sql);

		unset($this->user_preferences[$_name]);
	    }
	    else {
		$_value = $db->real_escape_string($value);
		$sql = "call set_preference('{$this->user_id}', '{$_name}', '{$_value}')";
		$db->mQuery($sql);
	    }
	}

	$pattern = '/^(.*)\/(\d*)$/i';

	$idx = preg_replace($pattern, '\2', $name);

	if (is_numeric($idx)) {
	    $this->user_preferences[preg_replace($pattern, '\1', $name)][$idx] = $value;
	}
	else {
	    $this->user_preferences[$name] = $value;
	}
    }
}
}
?>
