<?php
/*
 * © EUROPEAN COMMISSION, 2008-2010
 *
 * Concédée sous licence EUPL, version 1.1 uniquement (la «Licence»).
 * Vous ne pouvez utiliser la présente oeuvre que conformément à la Licence.
 * Vous pouvez obtenir une copie de la Licence à l’adresse suivante:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Sauf obligation légale ou contractuelle écrite, le logiciel distribué sous la Licence est distribué «en l’état»,
 * SANS GARANTIES OU CONDITIONS QUELLES QU’ELLES SOIENT, expresses ou implicites.
 * Consultez la Licence pour les autorisations et les restrictions linguistiques spécifiques relevant de la Licence.
 *
 * Auteurs de l'oeuvre:
 *   Pascal Havelange, Pascal.HAVELANGE@ec.europa.eu
 *   Cosmin POPESCU, Adrian.POPESCU@ext.ec.europa.eu
 *
 */

/**
	 * Small Helper Class to parse NITF documents
	 * @example examples/nitf_document.php Parse a document and displays it using the (implied) __toString method
	 * */
	class NITF_document {
		
		/**
		 * @var DOMDocument
		 * */
		protected $dom;
		
		/**
		 * @var DOMXPath
		 * */
		protected $xpath;
		
		// Docdata
		/**
		 * @var string
		 * */
		public $docdata_doc_id = null;
		/**
		 * @var string
		 * */
		public $docdata_doc_src = null;
		/**
		 * @var string
		 * */
		public $docdata_date_issue = null;
		/**
		 * @var string
		 * */
		public $docdata_copyright_holder = null;
		/**
		 * @var string
		 * */
		public $docdata_copyright_year = null;
		
		// Pubdata
		/**
		 * @var string
		 * */
		public $pubdata_unit_of_measure = null;
		/**
		 * @var string
		 * */
		public $pubdata_item_length = null;
		/**
		 * @var string
		 * */
		public $pubdata_name = null;
		/**
		 * @var string
		 * */
		public $pubdata_position_section = null;
		
		// Body/Head
		/**
		 * @var string
		 * */
		public $byline = null;
		/**
		 * @var string
		 * */
		public $headline = null;
		/**
		 * @var string
		 * */
		public $abstract = null;
		
		// Body/Content
		/**
		 * @var string
		 * */
		public $body = null;
		
		// Body/Notes
		/**
		 * @var string
		 * */
		public $notes = null;
		
		// Is XML from last 'load' operation valid?
		/**
		 * @var boolean
		 * */
		public $valid = false;
		
		/**
		 * Constructor
		 * @return void
		 * */
		public function __construct() {
			$this->dom = new DOMDocument();
			$this->dom->formatOutput = true;
		}
		
		/**
		 * Load XML from a file
		 * @param string $file File PAth
		 * @return boolean
		 * */
		public function load( $file ) {
			$this->valid = false;
			if( false === $this->dom->load( $file ) ) {
				return false;
			}
			if( false === $this->dom->validate() ) {
				return false;
			}
			$this->valid = true;
			return $this->parseDocument();
		}
		
		/**
		 * Load XML from a string
		 * @param string $xml NITF Data
		 * @return boolean
		 * */
		public function loadXML( $xml ) {
			$this->valid = false;
			if( false === $this->dom->loadXML( $xml ) ) {
				return false;
			}
			if( false === $this->dom->validate() ) {
				return false;
			}
			$this->valid = true;
			return $this->parseDocument();
		}
		
		/**
		 * Dumps the internal XML tree back into a string
		 * @return string
		 * */
		public function saveXML() {
			return $this->dom->saveXML();
		}
		/**
		 * Dumps the internal XML tree back into a file
		 * @param string $filename File Path
		 * @return boolean
		 * */
		public function save( $filename ) {
			return $this->dom->saveXML( $filename );
		}
		
		/**
		 * Basic output of the document
		 * @return string
		 * */
		public function __toString() {
			$abstract = "";
			if( is_string( $this->abstract ) && 0 != strlen( $this->abstract )) {
				$abstract = <<<txt
<h2>Abstract:</h2>
{$this->abstract}
<br/>

txt;
			}
			$body = "";
			if( is_string( $this->body ) && 0 != strlen( $this->body )) {
				$body = <<<txt
<h2>Body:</h2>
{$this->body}
<br/>

txt;
			}

			$out = <<<txt
<code>
DOC ID:  {$this->docdata_doc_id}<br/>
DOC SRC: {$this->docdata_doc_src}<br/>
DOC DATE OF ISSUE: {$this->docdata_date_issue}<br/>
PUBLICATION: {$this->pubdata_name}<br/>
POSITION:    {$this->pubdata_position_section}<br/>
SIZE:   {$this->pubdata_item_length} {$this->pubdata_unit_of_measure}<br/>
BYLINE: {$this->byline}<br/>
COPYRIGHT: {$this->docdata_copyright_holder}<br/>
</code>
<h1>{$this->headline}</h1>
<br/>
{$abstract}
{$body}

txt;
			if( null != $this->notes ) {
				foreach( $this->notes as $category => $value ) {
					$out .= "<em><strong>{$category}</strong>: {$value}</em><br/>\n";
				}
			}
			return $out;
		}
		
		/**
		 * Parse the loaded NITF document
		 * @return boolean
		 * */
		protected function parseDocument() {
			$rc = false;
			
			$this->xpath = new DOMXPath( $this->dom );
			
			// Parsing NITF/Head
			$xrs = $this->xpath->query( "/nitf/head/pubdata" );
			if( $xrs->length ) {
				if( $xrs->item(0)->hasAttribute( "unit-of-measure" )) {
					$this->pubdata_unit_of_measure = $xrs->item(0)->getAttribute( "unit-of-measure" );
				}
				if( $xrs->item(0)->hasAttribute( "item-length" )) {
					$this->pubdata_item_length = $xrs->item(0)->getAttribute( "item-length" );
				}
				if( $xrs->item(0)->hasAttribute( "position.section" )) {
					$this->pubdata_position_section = $xrs->item(0)->getAttribute( "position.section" );
				}
				if( $xrs->item(0)->hasAttribute( "name" )) {
					$this->pubdata_name = $xrs->item(0)->getAttribute( "name" );
				}
			}
			$xrs = $this->xpath->query( "/nitf/head/docdata/doc-id" );
			if( $xrs->length ) {
				if( $xrs->item(0)->hasAttribute( "id-string" )) {
					$this->docdata_doc_id = $xrs->item(0)->getAttribute( "id-string" );
				}
				if( $xrs->item(0)->hasAttribute( "regsrc" )) {
					$this->docdata_doc_src = $xrs->item(0)->getAttribute( "regsrc" );
				}
			}
			$xrs = $this->xpath->query( "/nitf/head/docdata/date.issue" );
			if( $xrs->length ) {
				if( $xrs->item(0)->hasAttribute( "norm" )) {
					$this->docdata_date_issue = $xrs->item(0)->getAttribute( "norm" );
				}
			}
			$xrs = $this->xpath->query( "/nitf/head/docdata/doc.copyright" );
			if( $xrs->length ) {
				if( $xrs->item(0)->hasAttribute( "holder" )) {
					$this->docdata_copyright_holder = $xrs->item(0)->getAttribute( "holder" );
				}
				if( $xrs->item(0)->hasAttribute( "year" )) {
					$this->docdata_copyright_year = $xrs->item(0)->getAttribute( "year" );
				}
			}
			
			
			// Parsing NITF/Body
			$xrs = $this->xpath->query( "/nitf/body/body.head/hedline" );
			if( $xrs->length ) {
				$this->headline = $xrs->item(0)->textContent;
			}
			$xrs = $this->xpath->query( "/nitf/body/body.head/byline" );
			if( $xrs->length ) {
				$this->byline = $xrs->item(0)->textContent;
			}
			$xrs = $this->xpath->query( "/nitf/body/body.head/abstract" );
			if( $xrs->length ) {
				$this->abstract = $xrs->item(0)->textContent;
			}

			$xrs = $this->xpath->query( "/nitf/body/body.content/block/abstract/*" );
			$this->abstract = self::getNodeListContent( $xrs, array("p","a","em","strong","quote","br","span","table","tr","th","td","tbody","tfoot","thead","li","ul","ol","img"), array("id", "href", "src", "name", "class", "style"));

			$xrs = $this->xpath->query( "/nitf/body/body.content/block/*[local-name() != 'abstract']" );
			$this->body = self::getNodeListContent( $xrs, array("p","a","em","strong","quote","br","span","table","tr","th","td","tbody","tfoot","thead","li","ul","ol","img"), array("id", "href", "src", "name", "class", "style"), array("abstract"));

			$xrs = $this->xpath->query( "/nitf/body/body.content/note" );
			if( $xrs->length ) {
				$this->notes = array();
			}
			for( $i = 0; $i < $xrs->length; $i ++ ) {
				if( $xrs->item($i)->hasAttribute( "class" )) {
					$this->notes[$xrs->item($i)->getAttribute( "class" )] = $xrs->item($i)->textContent;
				} else {
					$this->notes[] = $xrs->item($i)->textContent;
				}
			}
			
			return $rc;
		}
		
		/**
		 * Get a textual representation of a XML node (possibly keeping some predefined TAGS and ATTRIBUTES)
		 * @param DOMNodeList $nodeList
		 * @param string[] $tags_to_keep	A list of tags to be kept (for other tags, only the text content will be kept)
		 * @param string[] $attributes_to_keep	A list of attributes to be kept (other attributes will be discarded)
		 * @param string[] $childs_to_skip	A list of tag names that will be totally skipped (no tag, no content)
		 * */
		public static function getNodeListContent( $nodeList, $tags_to_keep = null, $attributes_to_keep = null, $childs_to_skip = null ) {
			$out = "";
			
			if( null == $tags_to_keep ) {
				$tags_to_keep = array();
			}

			if( null == $attributes_to_keep ) {
				$attributes_to_keep = array();
			}
			
			if( null == $childs_to_skip ) {
				$childs_to_skip = array();
			}
			
			for( $i = 0; $i < $nodeList->length; $i ++ ) {
				if( XML_TEXT_NODE == $nodeList->item($i)->nodeType )	 {
					$out .= $nodeList->item($i)->textContent;
				} else
				if( $nodeList->item($i)->hasChildNodes() ) {
					if( in_array( $nodeList->item($i)->nodeName, $childs_to_skip )) {
						continue;
					}
					if( in_array( $nodeList->item($i)->nodeName, $tags_to_keep )) {
						$out .= "<{$nodeList->item($i)->nodeName}";
						if( $nodeList->item($i)->hasAttributes() ) {
							for( $j = 0; $j < $nodeList->item($i)->attributes->length; $j ++ ) {
								if( in_array( $nodeList->item($i)->attributes->item( $j )->nodeName, $attributes_to_keep )) {
									$out .= " {$nodeList->item($i)->attributes->item( $j )->nodeName}=\"{$nodeList->item($i)->attributes->item( $j )->nodeValue}\"";
								}
							}
						}
						$out .= ">";
					}
					$out .= self::getNodeListContent( $nodeList->item($i)->childNodes, $tags_to_keep, $attributes_to_keep, $childs_to_skip );
					if( in_array( $nodeList->item($i)->nodeName, $tags_to_keep )) {
						$out .= "</{$nodeList->item($i)->nodeName}>";
					}
				} else {
					if( in_array( $nodeList->item($i)->nodeName, $childs_to_skip )) {
						continue;
					}
					if( in_array( $nodeList->item($i)->nodeName, $tags_to_keep )) {
						$out .= "<{$nodeList->item($i)->nodeName}";
						if( $nodeList->item($i)->hasAttributes() ) {
							for( $j = 0; $j < $nodeList->item($i)->attributes->length; $j ++ ) {
								if( in_array( $nodeList->item($i)->attributes->item( $j )->nodeName, $attributes_to_keep )) {
									$out .= " {$nodeList->item($i)->attributes->item( $j )->nodeName}=\"{$nodeList->item($i)->attributes->item( $j )->nodeValue}\"";
								}
							}
						}
						if( 0 == strlen( $nodeList->item($i)->nodeValue )) {
							$out .= "/>";
						} else {
							$out .= ">{$nodeList->item($i)->nodeValue}</{$nodeList->item($i)->nodeName}>";
						}
					} else {
						$out .= $nodeList->item($i)->nodeValue;
					}
				}
				
			}
			
			return $out;
		}
	}
?>