<?php
	/**
	 * Helper functions & classes for LDAP access 
	 * @author Pascal (Meta) Havelange (havelangep@gmail.com)
 	 * @copyright &copy; 2007 All rights reserved
 	 * @package MetaNull
 	 */	
	
	/**
	 * Base class for SMTP related functions
	 * @package MetaNull
	 */
	class smtpException extends Exception {
		public function __construct( $message, $code = 0) {
			parent::__construct( $message, $code );
		}
	}

	/**
	 * Exception indicating an invalid email address format
	 * @package MetaNull
	 */
	class invalidEmailAddressException extends smtpException {
		public function __construct( $message, $code = 0) {
			parent::__construct( $message, $code );
		}
	}

	
	/**
	 * Exception indicating an invalid attachment file
	 * @package MetaNull
	 */
	class invalidAttachmentException extends smtpException {
		public function __construct( $message, $code = 0) {
			parent::__construct( $message, $code );
		}
	}
	
	/**
	 * Helper Class for SMTP e-mails sending
	 * @author Pascal (Meta) Havelange (havelangep@gmail.com)
 	 * @copyright &copy; 2007 All rights reserved
 	 * @package MetaNull
	 */
	class mSmtp {
		
		/**
		 * Multidimentionnal array containing the list of recipient addresses ("to", "cc", "bcc",...)
		 * @access private 
		 * @var array
		 */
		private $addresses;
		
		/**
		 * Subject of the message
		 * @access private 
		 * @var string
		 */
		private $subject;
		
		/**
		 * The list of parts composing the body of the mail message
		 * @access private
		 * @var array
		 */
		private $body_parts;
		
		/**
		 * Unique boundary
		 * @access private
		 * @var string
		 */
		private $boundary;
		
		/**
		 * Priority level of the message
		 * @access private
		 * @var int
		 */
		private $priority;
		
		/**
		 * SMTP Server Host
		 * @access private
		 * @var string
		 */
		private $smtp_address;
		
		/**
		 * SMTP Server Port
		 * @access private
		 * @var string
		 */
		private $smtp_port = 25;
		
		const MMAIL_PRIORITY_LOWEST		= 5;		/**< e-mail lowest priority */
		const MMAIL_PRIORITY_LOW		= 4;		/**< e-mail low priority */
		const MMAIL_PRIORITY_NORMAL		= 3;		/**< e-mail normal priority */
		const MMAIL_PRIORITY_HIGH		= 2;		/**< e-mail high priority */
		const MMAIL_PRIORITY_HIGHEST	= 1;		/**< e-mail highest priority */
	
		const MMAIL_TYPE_TEXT			= "text/plain";		/**< The mail's content type is text */
		const MMAIL_TYPE_HTML			= "text/html";		/**< The mail's content type is HTML */
		const MMAIL_TYPE_IMAGE_JPG		= "image/jpg";		/**< The mail's content type is jpeg */
		const MMAIL_TYPE_IMAGE_GIF		= "image/gif";		/**< The mail's content type is gif */
	
		const MMAIL_ENCODING_UTF8		= "UTF-8";			/**< The mail's content encoding is utf-8 */
		const MMAIL_ENCODING_ISO88591	= "ISO-8859-1";		/**< The mail's content encoding is ISO-8859-1 */
	
		const MMAIL_TRANSFER_ENCODING_8BIT		= "8bit";	/**< The mail's encoding mechanism is '8 bit' */
		const MMAIL_TRANSFER_ENCODING_7BIT		= "7bit";	/**< The mail's encoding mechanism is '7 bit' */
		const MMAIL_TRANSFER_ENCODING_BASE64	= "base64";	/**< The mail's encoding mechanism is 'base 64' */
		
		/**
		 * Constructor
		 * @access public
		 * */
		public function __construct( $sender, $smtp_address, $smtp_port = 25) {
			$this->addresses = array( "from" => false, "return" => false, "to" => array(), "cc" => array(), "bcc" => array() );
			$this->addresses["to"] = array();
			$this->addresses["from"] = "";
			$this->addresses["return"] = "";
			$this->addresses["cc"] = array();
			$this->addresses["bcc"] = array();
			
			$this->body_parts = array();
			$this->body_parts[0] = false;
			
			$this->subject = false;
			$this->boundary = "-----=" . md5( uniqid( rand(), true ) );
			$this->priority = self::MMAIL_PRIORITY_NORMAL;
			
			$this->setFrom( $sender );
			$this->setReturnAddress( $sender );
			$this->setSMTPServer( $smtp_address, $smtp_port );
		}
		
		/**
		 * Set the SMTP Server Host
		 * @param string $smtp_address SMTP Server
		 * @final
		 * @access public
		 */
		final public function setSMTPServer( $smtp_address = "localhost", $smtp_port = 25 ) {
			$this->smtp_address = $smtp_address;
			$this->smtp_port = $smtp_port;
		}
		
		/**
		 * Set the address from which this e-mail wiil be send
		 * @param string $from_address E-mail address the recipient will see as beeing the sender of this e-mail
		 * @final
		 * @access public
		 */
		final public function setFrom( $from_address ) {
			if( false == mSmtp::testEmailAddress( $from_address ) ) {
				throw new invalidEmailAddressException( "The e-mail address `{$from_address}` do not seems to be valid!" );
			}
			$this->addresses["from"] = $from_address;
		}
		
		/**
		 * Set the address that the receiver will have to use to reply to your mail (its mail client will then ignore the 'from' address)
		 * @param string $return_address E-mail address the recipient should use to reply to the mail
		 * @final
		 * @access public
		 */
		final public function setReturnAddress( $return_address ) {
			if( false == mSmtp::testEmailAddress( $return_address ) ) {
				throw new invalidEmailAddressException( "The e-mail address `{$return_address}` do not seems to be valid!" );
			}
			$this->addresses["return"] = $return_address;
		}
		
		/**
		 * Set the list of recipients of the mail
		 * @param array|string $recipients The e-mail address of the recipient, or an array of email addresses
		 * @final
		 * @access public
		 */
		final public function setRecipients( $recipients ) {
			$this->setAddresses( $recipients, $this->addresses["to"] );
		}
		
		/**
		 * Add recipients to the mail
		 * @param array|string $recipients The e-mail address of the recipient, or an array of email addresses to add
		 * @final
		 * @access public
		 */
		final public function addRecipients( $recipients ) {
			$this->addAddresses( $recipients, $this->addresses["to"] );
		}
		
		/**
		 * Set the list of "carbon copy" recipients of the mail
		 * @param array|string $cc The e-mail address of the recipient, or an array of email addresses
		 * @final
		 * @access public
		 */
		final public function setCC( $cc ) {
			$this->setAddresses( $cc, $this->addresses["cc"] );
		}
		
		/**
		 * Add "carbon copy" recipients to the mail
		 * @param array|string $cc The e-mail address of the recipient, or an array of email addresses to add
		 * @final
		 * @access public
		 */
		final public function addCC( $cc ) {
			$this->addAddresses( $cc, $this->addresses["cc"] );
		}
		
		/**
		 * Set the list of "blind carbon copy" recipients of the mail
		 * @param array|string $bcc The e-mail address of the recipient, or an array of email addresses
		 * @final
		 * @access public
		 */
		final public function setBCC( $bcc ) {
			$this->setAddresses( $bcc, $this->addresses["bcc"] );
		}
		
		/**
		 * Add "blind carbon copy" recipients to the mail
		 * @param array|string $bcc The e-mail address of the recipient, or an array of email addresses to add
		 * @final
		 * @access public
		 */
		final public function addBCC( $bcc ) {
			return $this->addAddresses( $bcc, $this->addresses["bcc"] );
		}
		
		/**
		 * Set the subject of the message
		 * @param string $string The subject of the message
		 * @final
		 * @access public
		 */
		final public function setSubject( $string ) {
			$this->subject = $string;
		}
		
		/**
		 * Set the priority level of the message 
		 * @param int $priority Message's priority
		 * @see MMAIL_PRIORITY_LOWEST
		 * @see MMAIL_PRIORITY_LOW
		 * @see MMAIL_PRIORITY_NORMAL
		 * @see MMAIL_PRIORITY_HIGH
		 * @see MMAIL_PRIORITY_HIGHEST)
		 * @final
		 * @access public
		 */
		final public function setPriority( $priority = mSmtp::MMAIL_PRIORITY_NORMAL ) {
			$this->priority = $priority;
		}
		
		/**
		 * Set the body of the message
		 * @param string $content Content of the message
		 * @param string $type Mime type of the message (default is MMAIL_TYPE_HTML)
		 * @param string $encoding Character encoding to use (default is MMAIL_ENCODING_UTF8)
		 * @param string $xfertEncoding Content-Transfer-Encoding to use (default is MMAIL_TRANSFER_ENCODING_8BIT)
		 * @see MMAIL_TYPE_TEXT
		 * @see MMAIL_TYPE_HTML
		 * @see MMAIL_ENCODING_UTF8
		 * @see MMAIL_ENCODING_ISO88591
		 * @see MMAIL_TRANSFER_ENCODING_8BIT
		 * @see MMAIL_TRANSFER_ENCODING_7BIT
		 * @see MMAIL_TRANSFER_ENCODING_BASE64
		 * @final 
		 * @access public
		 */
		final public function setMessage( $content, $type = mSmtp::MMAIL_TYPE_HTML, $encoding = mSmtp::MMAIL_ENCODING_UTF8, $xfertEncoding = mSmtp::MMAIL_TRANSFER_ENCODING_8BIT ) {
			$this->body_parts[0] = array();
			$this->body_parts[0]["type"] = $type;
			$this->body_parts[0]["encoding"] = $encoding;
			$this->body_parts[0]["transferencoding"] = $xfertEncoding;
			$this->body_parts[0]["data"] = $content;
		}
		
		/**
		 * Add an attachment to the message
		 * @param string $file Either the Full path to the file to attach, either the data to attach itself
		 * @param string $name Filename to use in the message
		 * @param string $mime_type Mime type of the file (if not set application/octet-stream will be used)
		 * @param boolean $isData Flag indicating if (when True) $file contains the data to attach, or (when false) if it is the path to a file to attach.
		 * @param string $cid Attachment identifier (If not provided, it will be automatically generated)
		 * @return string The Attachment's ID. You can use it to embed the attachment in the body of your message, using the syntax <img src="cid:ATTACH_ID" />
		 * @access public
		 * @final
		 */
		final public function addAttachment( $file, $name = false, $mime_type = false, $isData = false, $cid = null ) {
			$part = array();
			$part["id"] = ((null == $cid) ? md5( uniqid( rand(), true ) ) : $cid);
			$part["type"] = $mime_type;
			$part["encoding"] = false;
			$part["name"] = $name;
			$part["transferencoding"] = self::MMAIL_TRANSFER_ENCODING_BASE64;
			
			$data = "";
			if( true == $isData ) {
				if( false === $mime_type ) {
					$part["type"] = "application/octet-stream";
				}
				$data = $file;
			} else {
				if( false === $mime_type ) {
					if( function_exists( "mime_content_type" ) ) {
						$part["type"] = mime_content_type( $file );
					} else {
						$part["type"] = "application/octet-stream";
					}
				}
				if( false === $name ) {
					$pi = pathinfo( $file );
					$part["name"] = $pi["basename"];
				}
				$real_path = realpath( $file );
				if( false == $real_path ) {
					throw new invalidAttachmentException( "Invalid attachment path: `{$real_path}`!" );
				}
				$fp = fopen( $real_path, "rb" );
				if( false === $fp ) {
					throw new invalidAttachmentException( "Error reading attachment: `{$real_path}`!" );
				}
				while( !feof( $fp )) {
					$data .= fread( $fp, 1024 );
				}
				fclose( $fp );
			}
			$part["data"] = chunk_split( base64_encode ( $data ));
			
			$this->body_parts[] = $part;
			
			return $part["id"];
		}
		
		/**
		 * Send the message
		 * @access public
		 * @final
		 */
		final public function send() {
			$mail_recipients = "";
			if( 0 < count( $this->addresses["to"] ) ) {
				$mail_targets_count = 0;
				foreach( $this->addresses["to"] as $to ) {
					if( 0 != $mail_targets_count ++ ) {
						$mail_recipients .= ",";
					}
					$mail_recipients .= $to;
				}
			} else {
				throw new smtpException( "The recipient's address is missing!" );
			}
			$mail_subject = "";
			if( false !== $this->subject ) {
				$mail_subject = $this->subject;
			}
			
			$mail_body = $this->buildContent();
			$mail_headers = $this->buildHeader();
			
			ini_set( "sendmail_from", $this->addresses["from"] );
			ini_set( "SMTP", $this->smtp_address );
			ini_set( "smtp_port", $this->smtp_port );
			
			$post_rc = @mail( $mail_recipients, $mail_subject, $mail_body, $mail_headers );
			
			ini_restore( "sendmail_from" );
			ini_restore( "SMTP" );
			ini_restore( "smtp_port" );

			if( false === $post_rc ) {
				throw new smtpException( "Error sending e-mail message to {$mail_recipients}!\n{$php_errormsg}" );
			}
		}
		
		/**
		 * Returns the body of the message
		 * @access private
		 * @final
		 * @return string
		 */
		final private function buildContent() {
			$mail_body = "";
			
			// Build Message Body:
			$mail_body .= "--" . $this->boundary . "\r\n";
			$body_part_count = 0;
			
			foreach( $this->body_parts as $part ) {
				if( 0 < $body_part_count++ ) {
					$mail_body .= "--" . $this->boundary . "\r\n";
				}
				$mail_body .= "Content-Type: ".$part["type"].";";
				if( isset( $part["encoding"] )) {
					$mail_body .= " charset=" . $part["encoding"] . ";";
				}
				if( isset( $part["name"] )) {
					$mail_body .= " name=\"" . $part["name"] . "\";";
				}
				$mail_body .= "\r\n";
				if( isset( $part["transferencoding"] )) {
					$mail_body .= "Content-Transfer-Encoding: ".$part["transferencoding"]."\r\n" ;
				}
				if( isset( $part["id"] )) {
					$mail_body .= "Content-Id: ". $part["id"] . "\r\n";
				}
				$mail_body .= "\r\n";
				$mail_body .= $part["data"] . "\r\n";
				$mail_body .= "\r\n";
			}
			$mail_body .= "--" . $this->boundary . "--\r\n";
			
			return $mail_body;
		}
		
		/**
		 * Returns the header of the message
		 * @access private
		 * @final 
		 * @return string
		 */
		final private function buildHeader() {
			$mail_headers = "";
			
			// Build Message Headers
			$mail_headers .= "MIME-Version: 1.0\r\n";
			$mail_headers .= "X-Sender: ".$this->addresses["from"]."\r\n";
			$mail_headers .= "X-Mailer: PHP/" . phpversion() . "\r\n";
			$mail_headers .= "X-Priority: ".sprintf( "%d", $this->priority )."\r\n";
			$mail_headers .= "Return-Path: ".$this->addresses["return"]."\r\n";
			$mail_headers .= "From: \"".$this->addresses["from"]."\" <".$this->addresses["from"].">\r\n";
			if( 0 < count( $this->addresses["cc"] ) ) {
				$mail_headers .= "cc:";
				$mail_targets_count = 0;
				foreach( $this->addresses["cc"] as $cc ) {
					if( 0 != $mail_targets_count ++ ) {
						$mail_headers .= ",";
					}
					$mail_headers .= $cc;
				}
				$mail_headers .= "\r\n";
			}
			if( 0 < count( $this->addresses["bcc"] ) ) {
				$mail_headers .= "bcc:";
				$mail_targets_count = 0;
				foreach( $this->addresses["bcc"] as $bcc ) {
					if( 0 != $mail_targets_count ++ ) {
						$mail_headers .= ",";
					}
					$mail_headers .= $bcc;
				}
				$mail_headers .= "\r\n";
			}
			
			$mail_headers .= "Content-Type: multipart/mixed; boundary=\"".$this->boundary."\"\r\n";
			$mail_headers .= "\r\n";
			
			return $mail_headers;
		}
		
		/**
		 * Set a list of addresses. If the target array was not empty it is first cleared
		 * @access private
		 * @final
		 * @param string|array $addresses The address or list of addresses to add
		 * @param array $array The target array
		 */
		final private function setAddresses( $addresses, &$array ) {
			$array = array();
			$this->addAddresses( $addresses, $array );
		}
		
		/**
		 * Add one or several addresses to a list of addresses
		 * @access private
		 * @final 
		 * @param string|array $addresses The address or list of addresses to add
		 * @param array $array The target array
		 */
		final private function addAddresses( $addresses, &$array ) {
			if( !is_array( $addresses ) ) {
				$this->addAddress( $addresses, $array );
			} else {
				foreach( $addresses as $address ) {
					$this->addAddress( $address, $array );
				}
			}
			return true;
		}
		
		/**
		 * Add an address to a list of addresses
		 * @access private
		 * @final
		 * @param string $address	The address to add
		 * @param array $array	The destination array
		 */
		final private function addAddress( $address, &$array ) {
			if( !mSmtp::testEmailAddress( $address ) ) {
				throw new invalidEmailAddressException( "The e-mail address `{$address}` is not valid!" );
			}
			$array[] = $address;
		}
		
		/**
		 * Test the format of an email address, according to RFC 2822 and 2821
		 * @param string $address
		 * @return boolean
		 */
		static function testEmailAddress( $address, $rfc_allowed_length = true, $rfc_allowed_characters = true ) {

			// An e-mail address must contain a local-part and a domain-part, separated by a '@' character
			// None of these two parts can be empty
			// The local-part of an email-address may be escaped
			$regs = false;
			$address_parts = array();
			$is_escaped = false;
			if( false !== ereg( "^\"(.*)\"@(.*)$", $address, $regs )) {
				$address_parts[0] = $regs[1];
				$address_parts[1] = $regs[2];
				$is_escaped = true;
			} else {
				$address_parts = explode( "@", $address );
			}
			$address_parts = array_filter( array_map( "trim", $address_parts ));
			if( 2 != count( $address_parts )) {
				return false;
			}
			list( $local_part, $domain_part ) = $address_parts;
			
			
			// The Local part:
			// - Must be 64 characters long at maximum
			if( $rfc_allowed_length && strlen( $local_part ) > 64 ) {
				return false;
			}
			
			// - If the local part is not escaped, it may only contain:
			//		# Uppercase and lowercase letters
			//		# The digits 0 through 9
			//		# The characters ! # $ % * / ? | ^ { } ` ~ & ' + - = _
			//		# The character . provided that it is not the first nor last character in the local-part, nor may it appear two or more times consecutively.
			//
			// - Note that the ! # $ % * / ? | ^ { } ` ~ character are forbidden by some smtp server (ie hotmail)
			
			// - The character '.' can't be the first nor last character in the local-part. It may not appear two or more times consecutively
			if( false !== ereg( "(^\\.|\\.\\.|\\.$)", $local_part ) ) {
				return false;
			}
			if( false === $is_escaped ) {
				if( $rfc_allowed_characters ) {
					if( false === ereg( "^[a-zA-Z0-9!#\\$%\\*/\\?\\|\\^\\{\\}`~&'+=_\\.-]+$", $local_part) ) {
						return false;
					}					
				} else {
					if( false === ereg( "^[a-zA-Z0-9&'+=_\\.-]+$", $local_part) ) {
						return false;
					}
				}
			} else {
				// The string is escaped (enclosed in double quotes ('"') characters, using the backslash character to escape invalid characters)
				// Therefore, the local_part should not ends by a single backslash ('\') character
				if( false !== ereg( "^.*[\\]$", $local_part ) && false === ereg( "^.*[\\][\\]$", $local_part ) ) {
					return false;
				}
				
				// The string is escaped, only test character that are not preceded by a '\' character
				$local_part_parts = split( "([\\].)", $local_part );
				foreach( $local_part_parts as $lp_part ) {
					if( $rfc_allowed_characters ) {
						if( false === ereg( "^[a-zA-Z0-9!#\\$%\\*/\\?\\|\\^\\{\\}`~&'+=_\\.-]+$", $lp_part) ) {
							return false;
						}
					} else {
						if( false === ereg( "^[a-zA-Z0-9&'+=_\\.-]+$", $lp_part) ) {
							return false;
						}
					}
				}
					
			}
			
			// The Domain part:
			// - Must be 255 characters long at maximum
			if( $rfc_allowed_length && strlen( $domain_part ) > 255 ) {
				return false;
			}
			// - The character '.' can't be the first nor last character in the domain-part. It may not appear two or more times consecutively
			if( false !== ereg( "(^\\.|\\.\\.|\\.$)", $domain_part ) ) {
				return false;
			}
			// - The domain name is much more restricted. The dot separated domain labels are limited to "letters, digits, and hyphens drawn from the ASCII character set ... Mailbox domains are not case sensitive."
			if( false === ereg( "^[a-zA-Z0-9\\.-]+$", $domain_part) ) {
				return false;
			}
			
			// The whole address:
			// - Must be 320 characters long at maximum
			//   BUT: However, there is a restriction in RFC 2821 on the length of an address in MAIL and RCPT commands of 256 characters
			//   THEREFORE: the upper limit on address lengths should normally be considered to be 256
			if( $rfc_allowed_length ) {
				if( strlen( "{$local_part}@{$domain_part}" ) > 256 ) {
					return false;
				}
			}
 
			return true;
		}
	
	};
	
?>